//
//  AudioRec.cpp
//  seq
//
//  Created by Jeffrey Harriman on 3/6/12.
//  Copyright (c) 2012 Stanford University. All rights reserved.
//

#include <iostream>
#include "AudioRecord.h"

AudioRecord::AudioRecord():Track(){
    m_ptr = 0;
    m_length = 0;
    // 10 second buffer
    m_recBuf = new SAMPLE[1];
    m_recBuf[0] = 0.0;
    setRecBufferSize(120*JIF_SRATE);
    setLoopLength(1*JIF_SRATE);
    m_active = true;
}

void AudioRecord::setRecBufferSize(int numFrames){
    // if less than 5 minutes
    if (numFrames < JIF_SRATE * 300){
    // set length variable
    m_maxLength = numFrames;
    m_length = numFrames;
    // throw out the old
    delete m_recBuf;
    // new
    m_recBuf = new SAMPLE [m_length];
    for (int i = 0; i < m_length; i ++){
        m_recBuf[i] = 0;
    }
    // reset pointer
    m_ptr = 0;
    }
}



void AudioRecord::Erase(){
    // clear record buffer
    setRecBufferSize(m_length);
    // reset the number of valid recordings
    m_numArts = 0;
    m_numNotes = 0;
    setPlayMode(STANDBY);
}


void AudioRecord::tick(Float32 *buffer, int numFrames){
    if (m_playMode == RECORD){ 
        for (int i = 0; i < numFrames; i++){
            // m_ptr points to the loop pointer variable
            // starting there increment up by numFrames to fill the buffer
            // this should be called before doing anything else to the audio buffer

            m_recBuf[m_ptr] =  buffer[2*i];
            // envelope
            buffer[2*i] = buffer[2*i+1] = m_adsr->tick() * buffer[2*i];
            m_ptr++;
            if (m_ptr >= m_length){
                setPlayMode(PLAY);
                m_metronome->setActive(false);
            }
        }
    }
    if (m_playMode == RECORD_ARTICULATION){ 
        for (int i = 0; i < numFrames; i++){
            buffer[2*i] = buffer[2*i+1] = m_adsr->tick() * m_recBuf[m_ptr];
            m_ptr++;
            if (m_ptr > m_length){
                setPlayMode(PLAY);
                m_metronome->setActive(false);
            }
        }
    }
    else if (m_playMode == LOOP){

        for (int i = 0; i < numFrames; i++){
            buffer[2*i] = buffer[2*i +1] = m_adsr->tick() * m_recBuf[m_ptr];
            m_ptr++;
            m_ptr %= m_length;
        }
    }
    else if (m_playMode == PLAY){
        
        for (int i = 0; i < numFrames; i++){
            // check if current frame lines up with a beat
            if (m_metronome->beginningOfBeat(i))
                // check for articulation data to play back
                playArticulation();
            
            buffer[2*i] = buffer[2*i +1] = m_adsr->tick() * m_recBuf[m_ptr]; 
            m_ptr++;
            m_ptr%=m_length;
            if (m_ptr >= m_maxLength){
                // ?
                setPlayMode(STANDBY);
            }
        }
    }
    else if (m_playMode == STANDBY){
        
        for (int i = 0; i < numFrames; i++){
            buffer[2*i] = buffer[2*i +1] = 0.0;
            
        }
    }

    else
        return;
}

audioRecording AudioRecord::getAudioRecording(){
    audioRecording trackData;
    trackData.length = m_length;
    trackData.audio_track = m_recBuf;
    
    return trackData;
}

void AudioRecord::loadAudioRecording(audioRecording downloadedRec){
    // unpack
    SAMPLE * buffer = downloadedRec.audio_track;
    m_length = downloadedRec.length;
    
    // copy the buffer 
    for (int i = 0; i < m_length; i++){
        m_recBuf[i] = buffer[i];
    }
    
    // reset pointers
    setPlayMode(STANDBY);
    
    return;
}