//
//  ChordTrack.cpp
//  tap_rec
//
//  Created by Jeffrey Harriman on 3/11/12.
//  Copyright (c) 2012 Stanford University. All rights reserved.
//

#include <iostream>
#include "ChordTrack.h"

ChordTrack::ChordTrack(): Track(){
    m_numVoices = 3;
    m_voices = new LeadTrack * [m_numVoices];
    for (int i = 0; i < m_numVoices; i++){
        m_voices[i] = new LeadTrack();
    }
    
    for (int i = 0; i < m_numVoices; i++){
        m_voices[i]->m_synth->setHarmonics(3);
        m_voices[i]->setOctave(-1);
        // will use the ADSR for this guy not the individual ones
        // same goes for record data
        m_voices[i]->keyOn();
        
    }
    setScaleDegree(0);
}

SAMPLE ChordTrack::tick(int frame){
    if (m_playMode == PLAY || m_playMode == LOOP){
        if (m_metronome->beginningOfBeat(frame))
            playArticulation();
    }
    
    if (m_playMode == PLAY || m_playMode == RECORD_ARTICULATION)
    {
        m_ptr++;
        if (m_ptr >= m_length){
            setPlayMode(PLAY);
            m_metronome->setActive(false);
        }
        
        playChordData();
    }
    else if (m_playMode == RECORD)
    {
        m_ptr++;
        if (m_ptr >= m_length){
            setPlayMode(PLAY);
            m_metronome->setActive(false);
        }
    }
    
    SAMPLE sum = 0;
    for (int i = 0; i < m_numVoices; i++){
        sum += .6 * m_voices[i]->tick(frame);
    }
    return m_adsr->tick() * sum;
}

//------------------------------------------
// setScaleDegree()
// 
//------------------------------------------
void ChordTrack::setScaleDegree(int scaleDegree){
    m_rootDegree = scaleDegree;
    
    if ((scaleDegree == ONE_MAJ) || (scaleDegree == FOUR_MAJ )|| (scaleDegree == FIVE_MAJ)){
        setMaj(scaleDegree);
    }
    else if( (scaleDegree == TWO_MAJ ) || (scaleDegree == THREE_MAJ ) || (scaleDegree == SIX_MAJ))
        setMin(scaleDegree);
    else if(scaleDegree == SEVEN_MAJ)
        setDim(scaleDegree);
    
    if (m_playMode == RECORD && m_keyName != scaleDegree){
        m_keyName = scaleDegree;
        // record note data:
        BEAT_COUNT timeStamp = m_metronome->getQuantizedBeat();
        recordData new_record;
        new_record.when = timeStamp;
        new_record.type = NOTE;
        new_record.value = (float)scaleDegree;
        recordNote(new_record);
    }
}

//------------------------------------------
// setMaj()
// Major chord
//------------------------------------------
void ChordTrack::setMaj(int scaleDegree){
    int root = scaleDegree;
    int third = root + 4;
    int fifth = root + 7;
    
    setChord(root, third, fifth);
    
}



//------------------------------------------
// setMin()
// Minor chord
//------------------------------------------
void ChordTrack::setMin(int scaleDegree){
    int root = scaleDegree;
    int third = root + 3;
    int fifth = root + 7;
    
    setChord(root, third, fifth);
    
}

//------------------------------------------
// SetDim()
// Diminished chord
//------------------------------------------
void ChordTrack::setDim(int scaleDegree){
    int root = scaleDegree;
    int third = root + 3;
    int fifth = root + 6;
    
    setChord(root, third, fifth);
    
}

//------------------------------------------
// SetChord()
// calculate frequencies and set the synthesizer as such
//------------------------------------------
void ChordTrack::setChord(int root, int third, int fifth){
    // within an octave
    root = convertScaleDegree(root % 12);
    third = convertScaleDegree(third % 12);
    fifth = convertScaleDegree(fifth % 12);
    
    
    
    int orderPitches[m_numVoices];
    
    if (root < third && root < fifth){
        orderPitches[0] = root;
        if (third < fifth){
            orderPitches[1] = third;
            orderPitches[2] = fifth;
        }
        else
        {
            orderPitches[1] = fifth;
            orderPitches[2] = third;
        }
        
    }
    else if ( third < root && third < fifth){
        orderPitches[0] = third;
        if (root < fifth){
            orderPitches[1] = root;
            orderPitches[2] = fifth;
        }
        else{
            orderPitches[1] = fifth;
            orderPitches[2] = root;
        }
    }
    
    else{
        orderPitches[0] = fifth;
        if (root < third){
            orderPitches[1] = root;
            orderPitches[2] = third;
        }
        else {
            orderPitches[1] = third;
            orderPitches[2] = root;
        }
    }
    
    
   // m_bass->setFrequency(midiToFreq(root - 24));
    
    //    
    for (int i = 0; i < m_numVoices; i ++){
        m_voices[i]->setNote(orderPitches[i]); //midiToFreq(orderPitches[i]);
    }
}

CHORD ChordTrack::getChord(){
    //    
    int triad[m_numVoices];
    for (int i = 0; i < m_numVoices; i ++){
        triad[i] = m_voices[i]->getNote() % 12;
    }
    CHORD triadChord;
    triadChord.root = convertScaleDegree(triad[0] % 12);
    triadChord.third = convertScaleDegree(triad[1] % 12);
    triadChord.fifth = convertScaleDegree(triad[2] % 12);
    
    return triadChord;
}


int ChordTrack::convertScaleDegree(int note){
    int majScale[7] = {ONE_MAJ, TWO_MAJ, THREE_MAJ, FOUR_MAJ, FIVE_MAJ, SIX_MAJ, SEVEN_MAJ};
    int degree = 0;
    for (int i = 0; i < 7; i++){
        if (note == majScale[i])
            degree = i;
    }
    
    return degree;
}


void ChordTrack::playChordData(){
    int initPtr;
    // while we have new note data
    BEAT_COUNT currentBeat = m_metronome->getQuantizedBeat();
    BEAT_COUNT nextNote = m_noteRec[m_notePtr].when;
//        NSLog(@"mtro: m: %i,b: %i,s: %i", currentBeat.measure, currentBeat.beat, currentBeat.subBeat);
//        NSLog(@"next: m: %i,b: %i,s: %i", nextNote.measure, nextNote.beat, nextNote.subBeat);
    while ((nextNote.measure == currentBeat.measure) && ( nextNote.beat == currentBeat.beat ) && ( nextNote.subBeat == currentBeat.subBeat ) )
    {
        if(m_noteRec[m_notePtr].type == NOTE){
            //NSLog(@"playing note data: %i", (int)m_noteRec[m_notePtr].value);
            setScaleDegree((int)(m_noteRec[m_notePtr].value));
            
        }
        
        m_notePtr++;
        m_notePtr %= m_numNotes;
        if (initPtr == m_notePtr)
            return;
        nextNote = m_noteRec[m_notePtr].when;
    }
}
