#import "mo_gfx.h"
#include <list>

#import "Accompaniment.h"


    
//-----------------------------------------------------------------------------
// name: class Entity
// desc: abstraction for a entity
//-----------------------------------------------------------------------------
class Entity
{
public:
    Entity() : alpha(1.0f) { }
    virtual void render() { /* default: do nothing */ }
    
    void set_color(Vector3D color){
        col=color;
    }
    void set_color(float x, float y, float z){
        col.x=x;
        col.y=y;
        col.z=z;
    }
    
    
    
public:
        // location
    Vector3D loc;
        // orientation
    Vector3D ori;
        // scaling
    Vector3D sca;
        // color
    Vector3D col;
        // alpha
    GLfloat alpha;
        // velocity
    Vector3D vel;
};

//-----------------------------------------------------------------------------
// name: class PolyEntity
// desc: polygon entity
//-----------------------------------------------------------------------------

class PolyEntity : public Entity
{
public:
    PolyEntity(): num_sides(24), radius(20) {}
    PolyEntity(int n_s, int r):num_sides(n_s),radius(r){}
    
    virtual void render();

    
public:
    int num_sides;
    int radius;
    
};

//-----------------------------------------------------------------------------
// name: class PolyEntity
// desc: polygon entity
//-----------------------------------------------------------------------------

class SquareEntity : public Entity
{
public:
    SquareEntity();
    
    virtual void render();
    bool isTouchWithin(Vector3D touch);
    void currentChord(CHORD currentChord);

public:
    int m_width;
    int m_keyLabel;
    
};

//-----------------------------------------------------------------------------
// name: class TouchEntity
// desc: touch entity
//-----------------------------------------------------------------------------
class TouchEntity : public PolyEntity
{
public:
    TouchEntity() : active(false), touch_ref(NULL), init_band_norm(0.0f){ }
    
public:
    GLboolean active;
    UITouch * touch_ref;
    // if this is the third touch, stores the dot product of [3rd_touch - 1st_touch]*[2nd_touch-1st]
    // used to determine if the band has been stretched
    GLint init_band_norm;
    double time_created;
    
};

// PolyEntity with a pulsing color and alpha controls to burst and dim
class PulseEntity : public PolyEntity
{
public:      
    // ugh, what a nasty init list
    PulseEntity() : base_alpha(0.7f),burst_alpha(1.0f),alpha_decay(300.0f),color_var(Vector3D(0.1f,0.1f,0.0f)),base_color(Vector3D(0.5f,0.5f,1.0f)),seed(rand())
    {
        alpha=base_alpha;
        //synth=NULL;
        m_accompaniment = new Accompaniment(1);

    }
    
    PulseEntity(int scaleDegree) : base_alpha( (float)(scaleDegree / 11.0)),burst_alpha(1.0f),alpha_decay(300.0f),color_var(Vector3D(0.1f,0.1f,0.0f)),base_color(Vector3D(0.5f,0.5f,1.0f)),seed(rand())
    {
        alpha=base_alpha;
        //synth=NULL;
        m_accompaniment = new Accompaniment(scaleDegree);
        
    }
    // deconstructor
    ~PulseEntity(){};

    
    void start_burst();
    void start_burst(int vel);
    void set_color(Vector3D c);
    void set_color(GLfloat x, GLfloat y, GLfloat z);
    
    void setScaleDegree(int scaleDegree){
        m_accompaniment->setScaleDegree(scaleDegree); 
        base_alpha = (float)((scaleDegree + 2) / 13.0);
    };

    void setAlpha(GLfloat newAlpha){
        alpha = newAlpha;
        base_alpha = newAlpha;
    }
    void currentScaleDegree(int scaleDegree);
    
    virtual void render();
    
public:
    GLfloat seed, burst_alpha,base_alpha,alpha_decay;
    // amplitudes for how much the color pulses
    Vector3D color_var,base_color;

    // A pulse entity has an accompaniment
    Accompaniment * m_accompaniment;
    
};

//-----------------------------------------------------------------------------
// name: class MotionEntity
// desc: entity that moves
//-----------------------------------------------------------------------------
class MotionEntity : public PolyEntity
{
    
public:
    MotionEntity(): active(false){}
    MotionEntity(TouchEntity t);
    
    void trigger_pulses();
    void trigger_pulses(int vel);
    
    // maybe at some point i should do all the motion shitz in here
//    virtual void render();
    
public:
    GLboolean active;
    std::list<PulseEntity *> pulses;
    GLint num_pulses;
};

