#import "mo_gfx.h"
#import "mo_audio.h"
#import "Entity.h"
#import <stdio.h>
#import <iostream>

    
void PolyEntity::render()
{
    static GLfloat* circle_verts  = new GLfloat[2*num_sides];
    for (int i=0;i<num_sides;++i){
        float angle=2*M_PI*i/num_sides;
        circle_verts[2*i]=radius*cosf(angle);
        circle_verts[2*i+1]=radius*sinf(angle);
    }
    
    // enable
    glEnable( GL_TEXTURE_2D); //raaaa!!!! the absence of this line ate my dinner.
    glEnableClientState( GL_VERTEX_ARRAY ); 
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    
    glPushMatrix();
    
    // color
    glColor4f( col.x, col.y, col.z, alpha );

    // move frame
    glTranslatef(loc.x, loc.y, loc.z );
    
    // vertex
    glVertexPointer( 2, GL_FLOAT, 0, circle_verts );
    glDrawArrays( GL_TRIANGLE_FAN, 0, num_sides);
    
    glPopMatrix();
    
    // disable
    glDisable(GL_BLEND);
    glDisableClientState( GL_VERTEX_ARRAY );
}

MotionEntity::MotionEntity(TouchEntity t){
    num_sides=t.num_sides;
    radius=t.radius;
    loc=t.loc;
    ori=t.ori;
    sca=t.sca;
    col=t.col;
    alpha=t.alpha;
    vel=t.vel;
}

// if there are any pulses in our list of pulses... trigger them
void MotionEntity::trigger_pulses(){
    trigger_pulses(64);
}

// passes on the midi velocity
void MotionEntity::trigger_pulses(int vel){
    //super ugly
    int to_play=rand()%pulses.size();
    int i=0;
    for (std::list<PulseEntity*>::iterator it=pulses.begin();it!=pulses.end();it++){
        if (to_play==i++)
            (**it).start_burst(vel);
    }
}


void PulseEntity::start_burst(){
    start_burst(64);
}

// override inhertied b/c color is constantly changing...
void PulseEntity::set_color(Vector3D c){
    base_color=c;    
}

void PulseEntity::set_color(GLfloat x, GLfloat y, GLfloat z){
    base_color=Vector3D(x,y,z);
}

void PulseEntity::render(){
    if (alpha>base_alpha){
        alpha-=(burst_alpha/alpha_decay);
    }
    //should never really execute this
    else
        alpha=base_alpha;
    
    
    //maybe pulsing too fast?
    double t=MoGfx::getCurrentTime(true)*3+seed;
    
    // pulse color
    col=Vector3D(base_color.x+color_var.x*sin(t),base_color.y+color_var.y*cos(t),base_color.z-color_var.z*sin(t));
    
    PolyEntity::render();
}


////////---------
SquareEntity::SquareEntity(){
}

void SquareEntity::render(){
    // enable
    glEnable( GL_TEXTURE_2D); 
    glEnableClientState( GL_VERTEX_ARRAY ); 
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    
    glPushMatrix();
    
    // color// alpha isn't working I think because of texture settings

    glColor4f( col.x, col.y, col.z, alpha );
    // move
    glTranslatef(loc.x, loc.y, 0.);
    
    // vertex
    static const GLfloat squareVertices[] = {
        0, 0,
        m_width, 0,
        0, m_width,
        m_width, m_width,
    };
    
//    static GLfloat* square_verts  = new GLfloat[8];
//    square_verts[0] = square_verts[1] = square_verts[3] = square_verts[6] =  0;
//    square_verts[2] = square_verts[4] = square_verts[5] = square_verts[7] = m_width;
    
    // draw square:
    glVertexPointer( 2, GL_FLOAT, 0, squareVertices );
    glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
    
    glPopMatrix();
    
    // disable
    glDisable(GL_BLEND);
    glDisableClientState( GL_VERTEX_ARRAY );
}

bool SquareEntity::isTouchWithin(Vector3D touch){
    int diff_x = touch.x - loc.x;
    int diff_y = touch.y - loc.y;
    if ((diff_x > 0 && diff_y > 0) && (diff_x < m_width && diff_y < m_width))
        return true;
    else
        return false;
}

void SquareEntity::currentChord(CHORD currentChord){
    if ((currentChord.root % 7)== m_keyLabel % 7)
        alpha = 1.0;
    else if ((currentChord.third % 7) == m_keyLabel % 7)
        alpha = .8;
    else if ((currentChord.fifth % 7)== m_keyLabel % 7)
        alpha = .6;
    else
        alpha = 0.3;
}



void PulseEntity::currentScaleDegree(int scaleDegree){
    
    if (m_accompaniment->getScaleDegree() == scaleDegree)
        if (m_accompaniment->getScaleDegree() == ONE_MAJ)
            setAlpha( 1.0f);
        else
            setAlpha(0.5f);
        else{
            float notLikely = 0.3;
            float probable = 0.9;
            float perhaps = 0.6;
            switch (scaleDegree)
            {
                case ONE_MAJ:
                    //setAlpha((float)((11.0 - m_accompaniment->getScaleDegree() + 4.0 ) / 15.0 ));
                    setAlpha(0.5);
                    break;
                    
                case TWO_MAJ:
                    if (m_accompaniment->getScaleDegree() == FIVE_MAJ)
                        setAlpha(probable);
                    else if (m_accompaniment->getScaleDegree() == SEVEN_MAJ)
                        setAlpha(perhaps);
                    else 
                        setAlpha(notLikely);
                    
                    break;
                    
                case THREE_MAJ:
                    if (m_accompaniment->getScaleDegree() == SIX_MAJ)
                        setAlpha(probable);
                    else if (m_accompaniment->getScaleDegree() == FOUR_MAJ)
                        setAlpha(perhaps);
                    else 
                        setAlpha(notLikely);
                    break;
                    
                case FOUR_MAJ:
                    if (m_accompaniment->getScaleDegree() == SEVEN_MAJ || m_accompaniment->getScaleDegree() == TWO_MAJ)
                        setAlpha(probable);
                    else if (m_accompaniment->getScaleDegree() == FIVE_MAJ)
                        setAlpha(perhaps);
                    else if (m_accompaniment->getScaleDegree() == ONE_MAJ)
                        setAlpha(.4);
                    else 
                        setAlpha(notLikely);
                    
                    break;
                    
                case FIVE_MAJ:
                    if (m_accompaniment->getScaleDegree() == ONE_MAJ)
                        setAlpha(probable);
                    else if (m_accompaniment->getScaleDegree() == SIX_MAJ)
                        setAlpha(perhaps);
                    else 
                        setAlpha(notLikely);
                    break;
                    
                    
                case SIX_MAJ:
                    if (m_accompaniment->getScaleDegree() == TWO_MAJ)
                        setAlpha(probable);
                    else if (m_accompaniment->getScaleDegree() == FOUR_MAJ)
                        setAlpha(perhaps);
                    else 
                        setAlpha(notLikely);
                    break;
                    
                case SEVEN_MAJ:
                    if (m_accompaniment->getScaleDegree() == FIVE_MAJ || m_accompaniment->getScaleDegree() == THREE_MAJ)
                        setAlpha(probable);
                    else if (m_accompaniment->getScaleDegree() == ONE_MAJ)
                        setAlpha(perhaps);
                    else 
                        setAlpha(notLikely);
                    break;
                    
            }
        }
}
