//
//  JifDefines.h
//  SSS
//
//  Created by Jeffrey Harriman on 3/8/12.
//  Copyright (c) 2012 Stanford University. All rights reserved.
//

#include "mo_def.h"

#ifndef SSS_JifDefines_h
#define SSS_JifDefines_h

#define JIF_SRATE 24000
#define FRAMESIZE 512
#define NUM_CHANNELS 2
#define MAX_TOUCHES 5


#define NOTE_OFFSET 60 // middle C


enum playMode {
    STANDBY,
    RECORD,
    RECORD_ARTICULATION,
    PLAY,
    LOOP
};

enum my_tracks{
    AUDIO_TRACK,
    AUDIO_TRACK_2,
    CHORD_TRACK,
    BASS_TRACK,
    LEAD_TRACK,
    METRO_TRACK,
    NUM_TRACKS
};

enum record_types{
    NOTE,
    KEY_ON,
    KEY_OFF,
};

struct BEAT_COUNT{
    int measure;
    int beat;
    int subBeat;
};

// a record of an event for playback 
// can be key on/off or note value
struct recordData {
    struct BEAT_COUNT when;
    int type;
    float value;
};

struct audioRecording{  // Do I need to know how many measures and tempo for these objects?
    SAMPLE * audio_track;
    int length;
};

struct synthRecording {
    struct recordData * note_data;
    int length; // number of elements stored
};

struct artRecording{
    struct recordData * art_data;
    int length;
};

// Audio track struct 
// holds the audio recording and the articulation data
struct audioTrack{
    struct audioRecording audio;
    struct artRecording art;
};

// synth track struct
// holds the synth notes recording and articulation data
struct synthTrack{
    struct synthRecording synth;
    struct artRecording art;
};

struct CHORD {
    int root;
    int third;
    int fifth;
};

enum  {
    ONE_MAJ = 0,
    TWO_MAJ = 2,
    THREE_MAJ = 4,
    FOUR_MAJ = 5,
    FIVE_MAJ = 7,
    SIX_MAJ = 9,
    SEVEN_MAJ = 11
};

// use to keep track of type of data being loaded
enum {
    AUDIO_REC_ONLY,
    NOTE_REC_ONLY,
    RHYTHM_ONLY,
    AUDIO_PLUS_RHYTHM,
    NOTE_PLUS_RHYTHM,
    SONG_SET
};


#endif
