//
//  Metro.cpp
//  seq
//
//  Created by Jeffrey Harriman on 2/27/12.
//  Copyright (c) 2012 Stanford University. All rights reserved.
//

#include <iostream>

#include "Metro.h"

Metro::Metro(){
    
    m_on = true;
    m_numBeats = 4;
    m_subDiv = 4;
    m_numMeasures = 4;
    setTempo(120.0);
    
    setTimeZero();
    
    m_resFilt = new ResonantLowPass();
    
    m_adsr = new stk::ADSR();
    m_adsr->setAllTimes(.001, .01, .5, .05);
}

SAMPLE Metro::tick(){
    if (m_on){
        float filteredNoise = m_resFilt->tick(.001 * (rand() % 1000 ) - 1);
        // accent down beat
        if (m_beatCount == 0)
            filteredNoise *= 2.0;
        
        if (m_sampleCount == m_beatLengthInSamples){ 
            m_sampleCount %= m_beatLengthInSamples;    
            
            m_subCount++;
            m_subCount %= m_subDiv;
            
            if (m_subCount == 0){
                m_adsr->keyOn();
                m_beatCount++;
                m_beatCount %= m_numBeats;
                if (m_beatCount == 0 && m_countIn)
                {
                    setTimeZero();
                    m_countIn = false;
                }
                else if (m_beatCount == 0){
                    m_measureCount++;
                    m_measureCount %= m_numMeasures; // if first recording set numMeasures here
                    m_resFilt->setFilt(2200, 3.0);
                }
                else
                    m_resFilt->setFilt(1500, 3.0);
            }
            
        }
        else if (m_sampleCount == (int) (.0005 * stk::SRATE) || m_subCount != 0){
            m_adsr->keyOff();
        }
        // increment sample counter
        m_sampleCount++;
        
        return m_adsr->tick() * filteredNoise;
    }
    
    // when off still count beats but don't make sound
    else
    {
        if (m_sampleCount == m_beatLengthInSamples){ 
            m_sampleCount %= m_beatLengthInSamples; 
            
            m_subCount++;
            m_subCount %= m_subDiv;
            
            if (m_subCount == 0){
                m_beatCount++;
                m_beatCount %= m_numBeats;
                if (m_beatCount == 0){
                    m_measureCount++;
                    m_measureCount %= m_numMeasures;
                }
            }
        }
        // increment sample counter
        m_sampleCount++;
           
        
        return 0.0;
    }
}

void Metro::startCountIn(){
    setTimeZero();
    m_countIn = true;
}

void Metro::setTimeZero(){
    m_measureCount = 0;
    m_beatCount = 0;
    m_subCount = 0;
    m_sampleCount = 0;
}

void Metro::stop(){
    m_on = false;
    setTimeZero();
}

void Metro::setTempo(float newTempo){
    if (newTempo > 0.0 && newTempo < 1000){
        m_tempo = newTempo * m_subDiv;
    }
    
    m_beatLengthInSamples = (int)((float) stk::SRATE * ( 60.0 / m_tempo ));
}

bool Metro::beginningOfLoop(){
    if (m_measureCount == 0 && m_sampleCount == 0)
        return true;
    else 
        return false;
}

bool Metro::beginningOfBeat(int frame){
    int tempCount = (m_sampleCount + frame) % m_beatLengthInSamples;
    
    if (tempCount == 0)
        return true;
    else 
        return false;
}

BEAT_COUNT Metro::getQuantizedBeat(){
    BEAT_COUNT currentBeat;
    currentBeat.measure = m_measureCount;
    currentBeat.beat = m_beatCount;
    currentBeat.subBeat = m_subCount;

    return currentBeat;
}



