//
//  MixMaster.h
//  SSS
//
//  Created by Jeffrey Harriman on 3/9/12.
//  Copyright (c) 2012 Stanford University. All rights reserved.
//

#ifndef SSS_MixMaster_h
#define SSS_MixMaster_h
#include "JifDefines.h"
#include "AudioRecord.h"
#include "LeadTrack.h"
#include "BassTrack.h"
#include "ChordTrack.h"
#include "Metro.h"
#include "mo_gfx.h"

class MixMaster {
    SAMPLE ** m_mixBufArray;
    Track ** m_track;
    int * m_currentTrack;
    Metro * m_metronome;
    
    SAMPLE * m_vertices;
    
public:
    MixMaster();
    ~MixMaster();
    void tick(SAMPLE * buffer, int bufferSize);
    void setLoopLength(int measures, int beatsPer, float tempo);
    void keyOn();
    void keyOff();
    void setNote(int note);
    
    int getChordRootDegree(){return ((ChordTrack *)m_track[CHORD_TRACK])->getScaleDegree();};
    CHORD getChord(){return ((ChordTrack *)m_track[CHORD_TRACK])->getChord();}
    int getKeyName(){return m_track[*m_currentTrack]->getKeyName();};
    // return the audioRecording data (pointer to rec Buffer and length)
    
    // networking
    audioRecording getAudioRecording(int track);
    int loadAudioRecording(audioRecording recording);
    int loadAudioRecording(int track, audioRecording recording);
    
    synthRecording getNoteRecording(int track);
    artRecording getArtRecording(int track);
    int loadNoteRecording(synthRecording recording);
    int loadNoteRecording(int track, synthRecording recording);
    int loadArtRecording(artRecording recording);
    int loadArtRecording(int track, artRecording recording);
    
    // button functions
    void Stop();
    void Play();
    void Record();
    void Dub();
    void Erase();
    // switch functions
    void setActive(bool value);
    void setPlayArticulation(bool value);
    
    // assign pointers
    void assignCurrentTrackPtr(int * currentTrack){m_currentTrack = currentTrack;};
    void assignVertPtr(SAMPLE * vertices){m_vertices = vertices;};
};

#endif
