//
//  Lead.h
//  tap_rec
//
//  Created by Jeffrey Harriman on 3/10/12.
//  Copyright (c) 2012 Stanford University. All rights reserved.
//

#ifndef tap_rec_MonophonicTrack_h
#define tap_rec_MonophonicTrack_h

#include "mo_def.h"
#include "Track.h"
#include "OnePole.h"
#include "MidiUtil.h"

/* 
 ::::INHERITS::::
 
 // member variables
 int m_playMode;
 int m_length;
 int m_maxLength;
 // level
 float m_gain;
 // envelope
 stk::ADSR * m_adsr;
 // time keeper
 Metro * m_metronome;
 int * m_key;
 
 // record data
 recordData * m_noteRec;
 RECORD_DATA * m_artRec;
 
 int m_ptr;
 int m_notePtr, m_numNotes;
 int m_artPtr, m_numArts;
 
 
public:
 void tick(SAMPLE * buffer, int numFrames);
 SAMPLE tick();
 void keyOn();
 void keyOff();
 
 // set functions
 void setGain(float gain){m_gain = gain;};
 void setLoopLength(int numSamples);
 void setLoopLength(int measures, int beatsPer, float tempo);
 void assignMetro(Metro * theMetronome){m_metronome = theMetronome;};
 void assignKeyPtr(int * theKey){m_key = theKey;};
 void setPlayMode(int mode);
 void setTimeZero();
 // get
 int getPlayMode(){return m_playMode;};
 
 // recording and playback
 void recordNote(RECORD_DATA);
 void recordArticulation(RECORD_DATA);
 void playArticulation();
 
 // public member vars
 bool m_active;
 */

class MonophonicTrack: public Track{
protected:
    // portamento time between notes
    stk::OnePole * m_portamento;
    // wah-like filter
    ResonantLowPass * m_resFilt;
    // current MIDI note (target to slide to)
    int m_note;
    int m_octave;
    // current note in Hz (actual value) this will be filtered by the OnePole m_portamento 
    float m_freq; 
    
public:
    MonophonicTrack();
    ~MonophonicTrack();
    
    void tick(SAMPLE * buffer, int numFrames);

    
    void playNoteData();
    // effecting sound
    void setNote(int note);
    void setOctave(int octave){m_octave = octave;};
    // when chord changes but finger doesn't move:
    void updateNote();
    
    void setFilt(float fc, float q){
        m_resFilt->setFilt(fc, q);
    }
    
    int getNote(){return m_note;};
};



#endif
