//
//  MonophonicTrack.cpp
//  tap_rec
//
//  Created by Jeffrey Harriman on 3/10/12.
//  Copyright (c) 2012 Stanford University. All rights reserved.
//

#include <iostream>
#include "MonophonicTrack.h"


MonophonicTrack::MonophonicTrack():Track(){
    m_portamento = new stk::OnePole(.999);
    m_resFilt = new ResonantLowPass();
    
    m_note = 0;
    m_keyName = -1;
    m_freq = 0.0; 
    m_octave = 0;
}


MonophonicTrack::~MonophonicTrack(){

}

void MonophonicTrack::playNoteData(){
    int initPtr;
    // while we have new note data
    BEAT_COUNT currentBeat = m_metronome->getQuantizedBeat();
    BEAT_COUNT nextNote = m_noteRec[m_notePtr].when;
//    NSLog(@"mtro: m: %i,b: %i,s: %i", currentBeat.measure, currentBeat.beat, currentBeat.subBeat);
//    NSLog(@"next: m: %i,b: %i,s: %i", nextArt.measure, nextArt.beat, nextArt.subBeat);
    while ((nextNote.measure == currentBeat.measure) && ( nextNote.beat == currentBeat.beat ) && ( nextNote.subBeat == currentBeat.subBeat ) )
    {
        if(m_noteRec[m_notePtr].type == NOTE){
            //NSLog(@"playing note data: %i", (int)m_noteRec[m_notePtr].value);
            setNote((int)(m_noteRec[m_notePtr].value));
        
        }
                        
        m_notePtr++;
        m_notePtr %= m_numNotes;
        if (initPtr == m_notePtr)
            return;
        nextNote = m_noteRec[m_notePtr].when;
    }
}

//----------------------------------
// setNote()
// set target Midi Note
//----------------------------------
void MonophonicTrack::setNote(int note)
{
        int majScale[7] = {ONE_MAJ, TWO_MAJ, THREE_MAJ, FOUR_MAJ, FIVE_MAJ, SIX_MAJ, SEVEN_MAJ};
    
        int scaleDegree = note % 7;
        int octave = note / 7;
        octave *= 12;
        m_note = octave + majScale[scaleDegree] + NOTE_OFFSET + 12 * m_octave; // + *m_key 
        m_freq = midiToFreq(m_note);
    
    if (m_playMode == RECORD && m_keyName != note){
        m_keyName = note;
        // record note data:
        BEAT_COUNT timeStamp = m_metronome->getQuantizedBeat();
        recordData new_record;
        new_record.when = timeStamp;
        new_record.type = NOTE;
        new_record.value = (float)note;
        recordNote(new_record);
    }

};


