//
//  ResonantLowPass.h
//  tap a tune
//
//  Created by Jeffrey Harriman on 2/24/12.
//  Copyright (c) 2012 Stanford University. All rights reserved.
//

#ifndef tap_a_tune_ResonantLowPass_h
#define tap_a_tune_ResonantLowPass_h

#include "mo_def.h"
#include "Stk.h"

class ResonantLowPass {
    
    
    float m_Fc;
    float m_Q;
    float m_Gain;
    float m_drive;
    
    // signal processing functions
	struct Biquad {
		//  biquad filter section
		double	b0, b1, b2, a1, a2, z1, z2;
		
		Biquad() {
			this->b0=1.0;
			this->b1=0.0;
			this->b2=0.0;
			this->a1=0.0;
			this->a2=0.0;
			Reset();
		}
		void SetCoefs(double* coefs) {
			// set filter coefficients [b0 b1 b2 a1 a2]
			this->b0=*(coefs);
			this->b1=*(coefs+1);
			this->b2=*(coefs+2);
			this->a1=*(coefs+3);
			this->a2=*(coefs+4);
		}
		void Reset() {
			// reset filter state
			z1=0;
			z2=0;
		}
		void Process (SAMPLE input, SAMPLE& output) {
			// process input sample, direct form II transposed
			output = z1 + input*b0;
			z1 = z2 + input*b1 - output*a1;
			z2 = input*b2 - output*a2;
		}
	};
    
    double m_lpCoefs[5];	// filter coefficients
	Biquad m_lpFilter;		// filter
    
    
    
public:
    ResonantLowPass();
    ~ResonantLowPass();
    SAMPLE tick(SAMPLE input);
    void tick(SAMPLE * buffer, int numFrames);

    void bilinearTranform(double acoefs[], double dcoeffs[]);
	void designResonantLowPass(double* dcoefs, double center, double qval);
    float knob2value(float knob, const float *limits, float taper);
    
    void setFilt(float fc, float q){
        m_Fc = fc;
        m_Q = q;
        
        designResonantLowPass(m_lpCoefs, m_Fc, m_Q);
        m_lpFilter.SetCoefs(m_lpCoefs);
    }
};

// input filter gain limits, dB; taper, exponent
const static float GainLimits[2] = {-24.0, 24.0};
const static float GainTaper = 1.0;

// input filter center frequency limits, hz; taper, exponent
const static float FcLimits[2] = {50.0, 8000.0};
const static float FcTaper = -1.0;

// input filter resonance limits, dB; taper, exponent
const static float QLimits[2] = {0.25, 32.0};
const static float QTaper = -1.0;

enum{
    PARAM_FC,
    PARAM_Q,
    PARAM_GAIN
};

#endif
