//
//  ResonantLowPass.cpp
//  tap a tune
//
//  Created by Jeffrey Harriman on 2/24/12.
//  Copyright (c) 2012 Stanford University. All rights reserved.
//

#include <iostream>
#include "ResonantLowPass.h"

ResonantLowPass::ResonantLowPass(){
    m_Gain = 1.0; //knob2value(0.0, GainLimits, GainTaper);
    m_Fc = 2000.0; // knob2value(1000.0, FcLimits, FcTaper);
    m_Q = 10.0; // knob2value(5.0, QLimits, QTaper);
    
    // design new input filter
	designResonantLowPass(m_lpCoefs, m_Fc, m_Q);
	m_lpFilter.SetCoefs(m_lpCoefs);
}

//-----------------------------------------------------------------------------------------
SAMPLE ResonantLowPass::tick(SAMPLE filtIn){

	//update values inside for loop to avoid clicks (implement the leaky integrator in here)
        
		// add noise to input to prevent denormal
		SAMPLE output;
		// apply input gain, input filter
		m_lpFilter.Process(filtIn, output);  //here
        
		return 2.0*output;
}


//-----------------------------------------------------------------------------------------
void ResonantLowPass::bilinearTranform(double acoefs[], double dcoefs[])
{
	double b0, b1, b2, a0, a1, a2;		//storage for continuous-time filter coefs
	double bz0, bz1, bz2, az0, az1, az2;	// coefs for discrete-time filter.
	
	// For easier looking code...unpack
	b0 = acoefs[0]; b1 = acoefs[1]; b2 = acoefs[2]; a0 = acoefs[3]; a1 = acoefs[4]; a2 = acoefs[5];
	
	
	// apply bilinear transform
	bz0 = (4*b2*stk::SRATE * stk::SRATE + 2*b1*stk::SRATE + b0)/(4*a2*stk::SRATE*stk::SRATE+2*a1*stk::SRATE+a0);
	bz1 = (2*b0-8*b2*stk::SRATE*stk::SRATE)/((4*a2*stk::SRATE*stk::SRATE+2*a1*stk::SRATE+a0));
	bz2 = (4*b2*stk::SRATE*stk::SRATE-2*b1*stk::SRATE+b0)/((4*a2*stk::SRATE*stk::SRATE+2*a1*stk::SRATE+a0));
	az0 = 1; 
	az1 = (2*a0-8*a2*stk::SRATE*stk::SRATE)/((4*a2*stk::SRATE*stk::SRATE+2*a1*stk::SRATE+a0)); 
	az2 = (4*a2*stk::SRATE*stk::SRATE-2*a1*stk::SRATE+a0)/((4*a2*stk::SRATE*stk::SRATE+2*a1*stk::SRATE+a0));
	
	// replace coefficients to output
	dcoefs[0] = bz0; dcoefs[1] = bz1; dcoefs[2] = bz2; dcoefs[3] = az1; dcoefs[4] = az2;
    
}

//-----------------------------------------------------------------------------------------
void ResonantLowPass::designResonantLowPass(double* dcoefs, double center, double qval)
// design parametric filter based on input center frequency, gain, Q and sampling rate
{
	double b0, b1, b2, a0, a1, a2;		//storage for continuous-time filter coefs
	double acoefs[6];
    
	//Design parametric filter here. Filter should be of the form
	//
	//    2
	// b2s  + b1s + b0
	// ---------------
	//    2
	// a2s  + a1s + a0
	//
	// Parameters are center frequency in Hz, gain in dB, and Q.
	
 	///////////////START//////////////////
	center = center*2*M_PI;
	float gamma = 1;
	//for peaking filter use:
	b0 = 0.0; // 
	b1 = gamma/center;   	
	b2= 0.0;
	// for high pass form use:
	//b0 = 0.0;
	//b1 = 0;   
	//b2=gamma/(center*center); // gamma / wc^2
	a0 = 1;
	a1 = 1/(center*qval);   // (1/wc)/Q
	a2 = 1/(center*center); // 1/wc^2
	
	////////////////END/////////////////////	
	// pack the analog coeffs into an array and apply the bilinear tranform
	acoefs[0] = b0; acoefs[1] = b1; acoefs[2] = b2; acoefs[3] = a0; acoefs[4] = a1; acoefs[5] = a2;
	
	// inputs the 6 analog coeffs, output the 5 digital coeffs
	bilinearTranform(acoefs, dcoefs);
}


//-----------------------------------------------------------------------------------------
float ResonantLowPass::knob2value(float knob, const float *limits, float taper)
// convert knob on [0,1] to value in [limits[0],limits[1]] according to taper
{
	float value;
    
	if (taper > 0.0) {
		// algebraic taper
		value = limits[0] + (limits[1] - limits[0]) * pow(knob, taper);
	} else {
		// exponential taper
		value = limits[0] * exp(log(limits[1]/limits[0]) * knob);
	}
    
	return value;
};