//
//  Entities.h
//  SSS
//
//  Created by Jeffrey Harriman on 1/23/12.
//  Copyright (c) 2012 Stanford University. All rights reserved.
//

#ifndef SSS_Entities_h
#define SSS_Entities_h

#include "mo_gfx.h"

//-----------------------------------------------------------------------------
// name: class Entity
// desc: abstraction for a entity
//-----------------------------------------------------------------------------
class Entity
{
public:
    Entity() : alpha( 1.0f ) { }
    virtual void render() { /* default: do nothing */ }
    
public:
    // location
    Vector3D loc;
    // orientation
    Vector3D ori;
    // scaling
    Vector3D sca;
    // color
    Vector3D col;
    // alpha
    GLfloat alpha;
    // velocity
    Vector3D vel;
};


//-----------------------------------------------------------------------------
// name: class TouchEntity
// desc: touch entity
//-----------------------------------------------------------------------------
class TouchEntity : public Entity
{
public:
    TouchEntity() : active(false), touch_ref(NULL) { }
    virtual void render();
    
public:
    GLboolean active;
    UITouch * touch_ref;
};




//----------------------------------------------------------------------------
// name: render()
// desc: draw a touch entity
//----------------------------------------------------------------------------
void TouchEntity::render()
{
    static const GLfloat half_width = 30;
    static const GLfloat squareVertices[] = {
        -half_width, -half_width,
        half_width, -half_width,
        -half_width, half_width,
        half_width, half_width,
    };
    
    // enable
    glEnableClientState( GL_VERTEX_ARRAY );	
    
    glPushMatrix();
    
    // color
    glColor4f( col.x, col.y, col.z, alpha );
    // vertex
    glVertexPointer( 2, GL_FLOAT, 0, squareVertices );
    
    // triangle strip
    glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
    
    glPopMatrix();
    
    // disable
    glDisableClientState( GL_VERTEX_ARRAY );
}


#endif
