//-----------------------------------------------------------------------------
//  name: renderer.mm
//  name: graphics + audio for HellOGLES
//
//  author: Ge Wang
//  date: winter 2011
//-----------------------------------------------------------------------------
#import "renderer.h"
#import "mo_audio.h"
#import "mo_gfx.h"

#import "mo_touch.h"
#import <iostream>
#import "../Entities.h"


// defines
#define SRATE 24000
#define FRAMESIZE 512
#define NUM_CHANNELS 2
#define MAX_TOUCHES 5

// global
SAMPLE g_width = 1.5;
GLint g_GfxWidth = 320;
GLint g_GfxHeight = 480;

// buffer
SAMPLE g_vertices[FRAMESIZE*4]; // made to be 4 times requested frame size since it is non-deterministic
// last frame size
UInt32 g_lastNumFrames = 0;

// touch entities!
TouchEntity * g_touches[MAX_TOUCHES];
// num of active touches
UInt32 g_numActiveTouches = 0;

//Touch_Callback 
void touch_callback( NSSet * , UIView * , const std::vector<MoTouchTrack> & tracks, void * );


//-----------------------------------------------------------------------------
// name: audio_callback()
// desc: audio callback, yeah
//-----------------------------------------------------------------------------
void audio_callback( Float32 * buffer, UInt32 numFrames, void * userData )
{
    g_lastNumFrames = numFrames;
    SAMPLE x = 0;
    SAMPLE inc = g_width / numFrames;
    
    for( int i = 0; i < numFrames; i++ )
    {
        // x coordinate
        g_vertices[2*i] = x; x += inc;
        // y coordinate
        g_vertices[2*i+1] = buffer[2*i] * 3;
        
//        // x coordinate
//        g_vertices[2*i] = buffer[2*i] * 3;
//        // y coordinate
//        g_vertices[2*i+1] = x; x += inc;
//        // zero
        buffer[2*i] = buffer[2*i+1] = 0;
    }
}




//-----------------------------------------------------------------------------
// name: setDims()
// desc: set dimension
//-----------------------------------------------------------------------------
void yaySetDims( GLint width, GLint height )
{
    g_GfxWidth = width;
    g_GfxHeight = height;
}




//-----------------------------------------------------------------------------
// name: init()
// desc: initialize graphics and sound
//-----------------------------------------------------------------------------
bool yayInit()
{
    // init touch before audio here:
    MoTouch::addCallback( touch_callback, NULL );
    
    // instantiate
    for (int i = 0; i < MAX_TOUCHES; i++)
    {
        g_touches[i] = new TouchEntity();
        g_touches[i] ->col.set(1,1,1);
    }
    
    // init
    bool result = MoAudio::init( SRATE, FRAMESIZE, NUM_CHANNELS );
    if( !result )
    {
        // do not do this:
        int * p = 0;
        *p = 0;
    }
    // start
    result = MoAudio::start( audio_callback, NULL );
    if( !result )
    {
        // do not do this:
        int * p = 0;
        *p = 0;
    }
	
	return true;
}




//-----------------------------------------------------------------------------
// name: cleanup()
// desc: clean up
//-----------------------------------------------------------------------------
void yayCleanup()
{}

///
//-----------------------------------------------------------------------------
// name: drawTouchEntities()
// desc: renders each of the touch entities
//-----------------------------------------------------------------------------
void drawTouchEntities()
{
    for( int i = 0; i < MAX_TOUCHES; i++ )
    {
        // check active
        if( g_touches[i]->active )
        {
            // push
            glPushMatrix();
            // translate
            glTranslatef( g_touches[i]->loc.x, g_touches[i]->loc.y, g_touches[i]->loc.z );
            // render
            g_touches[i]->render();
            // pop
            glPopMatrix();
        }
    }	
}


//-----------------------------------------------------------------------------
// name: drawLineBetween2Touches()
// desc: draws a line segment between the first two touches
//-----------------------------------------------------------------------------
void drawLineBetween2Touches()
{
    // check
    if( g_numActiveTouches != 2 ) return;
    
    // sanity check
    assert( g_touches[0]->active && g_touches[1]->active );
    
    float x0 = g_touches[0]->loc.x;
    float y0 = g_touches[0]->loc.y;
    float x1 = g_touches[1]->loc.x;
    float y1 = g_touches[1]->loc.y;
    
    // draw straight line from v0 to v1
    GLfloat lineVertices[4] = { x0, y0, x1, y1 };
    // enable
    glEnableClientState( GL_VERTEX_ARRAY );	
    
    // distance // - is overloaded to calc pythag
    GLfloat dist = (g_touches[0]->loc - g_touches[1]->loc).magnitude();
    

    
    // vertex
    glVertexPointer(2, GL_FLOAT, 0, lineVertices);
    
    // color
    glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
    // width
    GLfloat width = 1000 * 1./ (dist +1);
  
    //cerr << width << endl;;
    //glLineWidth(2.0);
    glLineWidth(width);
    
    // draw!
    glDrawArrays( GL_LINE_STRIP, 0, 2 );

}

//-----------------------------------------------------------------------------
// name: drawLineBetween3Touches()
// desc: draws a line segment between the first two touches
//-----------------------------------------------------------------------------
void drawLineBetween3Touches()
{
    // check
    if( g_numActiveTouches < 3 ) return;
    
    // sanity check
    assert( g_touches[0]->active && g_touches[1]->active );
    
    float x0 = g_touches[0]->loc.x;
    float y0 = g_touches[0]->loc.y;
    float x1 = g_touches[1]->loc.x;
    float y1 = g_touches[1]->loc.y;
    float x2 = g_touches[2]->loc.x;
    float y2 = g_touches[2]->loc.y;
    
    // draw straight line from v0 to v2
    GLfloat lineVertices[4] = { x0, y0, x2, y2 };
    // draw straight line from v1 to v2
    GLfloat lineVertices2[4] = { x1, y1, x2, y2 };
    // enable
    glEnableClientState( GL_VERTEX_ARRAY );	
    
    // distance // - is overloaded to calc pythag
    GLfloat dist = (g_touches[0]->loc - g_touches[2]->loc).magnitude() + (g_touches[1]->loc - g_touches[2]->loc).magnitude();
        
    // vertex
    glVertexPointer(2, GL_FLOAT, 0, lineVertices);
    glVertexPointer(2, GL_FLOAT, 0, lineVertices2);
    
    // color
    glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
    // width
    GLfloat width = 1000 * 1./ (dist +1);

    //cerr << width << endl;;
    //glLineWidth(2.0);
    glLineWidth(width);
    
    // draw!
    //glDrawArrays( GL_LINE_STRIP, 0, 2 );
    glDrawArrays( GL_LINE_STRIP, 0, 4 );
    
}


///

//-----------------------------------------------------------------------------
// name: render()
// desc: render next frame of graphics
//-----------------------------------------------------------------------------
//void yayRender()
//{
//    glMatrixMode(GL_PROJECTION);
//    glLoadIdentity();
//    // "portait mode"
//    glOrthof(-1.0, 1.0f, -1.5f, 1.5f, -1.0f, 1.0f);
//    glLoadIdentity();
//    
//    glClearColor(0.0f, 0.1f, 0.0f, 1.0f);
//    glClear(GL_COLOR_BUFFER_BIT);
//    
//    // rotate around z
//    glRotatef( 90, 0, 0, 1);
//
//    // center
//    glTranslatef(-g_width/2, 0, 0);
//    
//    glVertexPointer( 2, GL_FLOAT, 0, g_vertices);
//    //enable it when you want to use it, then drawArrays
//    glEnableClientState( GL_VERTEX_ARRAY);
//    
//    glColor4f(1.0, 1.0f, 1.0f, 1.0f);
//    
//    // g_lastNumFrames is how many to draw, if you do /2 it will just draw half the buffer
//    // only have line_strip, loop? and triangle strip?
//    glDrawArrays(GL_LINE_STRIP, 0, g_lastNumFrames);
//    
//    glDisableClientState(GL_VERTEX_ARRAY);
//    
//}
//-----------------------------------------------------------------------------
// name: render()
// desc: render next frame of graphics
//-----------------------------------------------------------------------------
void yayRender()
{
    // set to ortho
    //MoGfx::ortho( 480, 320, 0 );
    // this works for portrait mode:
    MoGfx::ortho( 320, 480, 1 );
    
    // set clear color
    glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
    // clear it
    glClear( GL_COLOR_BUFFER_BIT );
    
    // draw touches
    drawTouchEntities();
    // draw lines
    drawLineBetween2Touches();
    drawLineBetween3Touches();
}

//-----------------------------------------------------------------------------
// name: touch_callback()
// desc: touch callback function
//-----------------------------------------------------------------------------
void touch_callback( NSSet * touches, UIView * view, const std::vector<MoTouchTrack> & tracks, void * data)
{
    // iterate over touch points
    CGPoint location;
    for( UITouch * touch in touches )
    {
        // get the location
        location = [touch locationInView:nil];
        
        // transform: to make landscape
        double temp = location.x;
        location.x = location.y;
        location.y = temp;
        
        // NSLog( @"touch: %f, %f,", location.x, location.y );
        
        if( touch.phase == UITouchPhaseBegan )
        {
            // find idle touch entity
            TouchEntity * entity = NULL;
            for( int i = 0; i < MAX_TOUCHES; i++ )
            {
                // in case touch already active
                if( g_touches[i]->touch_ref == touch )
                    break;
                
                // find the next non-active touch
                if( !g_touches[i]->active )
                {
                    entity = g_touches[i];
                    break;
                }
            }
            
            // sanity check
            if( entity != NULL )
            {
                // set
                entity->active = true;
                entity->touch_ref = touch;
                entity->loc.x = location.x;
                entity->loc.y = location.y;                
                // count
                g_numActiveTouches++;
                // log it
                NSLog( @"active touches: %d", g_numActiveTouches );
            }
        }
        else if( touch.phase == UITouchPhaseMoved )
        {
            for( int i = 0; i < MAX_TOUCHES; i++ )
            {
                if( g_touches[i]->touch_ref == touch )
                {
                    g_touches[i]->loc.x = location.x;
                    g_touches[i]->loc.y = location.y;
                    break;
                }
            }
        }
        else if( (touch.phase == UITouchPhaseEnded) || (touch.phase == UITouchPhaseCancelled) )
        {
            for( int i = 0; i < MAX_TOUCHES; i++ )
            {
                if( g_touches[i]->touch_ref == touch )
                {
                    // set
                    g_touches[i]->active = false;
                    g_touches[i]->touch_ref = NULL;
                    
                    // pack active touches
                    for( int j = i+1; j < MAX_TOUCHES; j++ )
                    {
                        if( g_touches[j]->active )
                        {
                            // swap
                            TouchEntity * swap = g_touches[i];
                            g_touches[i] = g_touches[j];
                            g_touches[j] = swap;
                            // dangerous: advance i
                            i = j;
                        }
                    }
                    
                    // count
                    g_numActiveTouches--;
                    // log
                    NSLog( @"active touches: %d", g_numActiveTouches );
                    
                    break;
                }
            }
        }
    }
}


