//  SSSViewController.m
//  HellOGLES
//
//  Created by Ge Wang on 1/19/11.
//  Copyright 2011 Stanford University. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>

#import "SSSViewController.h"
#import "EAGLView.h"

#import "SongShareClient.h"
#import "AFHTTPRequestOperation.h"
#import <CoreLocation/CoreLocation.h>
#import "NSString+UUID.h"

#import "renderer.h"
#import "JifDefines.h"

#import "ModalTest.h"

int g_loadType;


@interface SSSViewController ()
{
    SongShareClient *theClient;
    CLLocationManager *locationManager;
    
    ModalTest *modalTest;
}

@property (nonatomic, strong) EAGLContext *context;
@property (nonatomic, strong) CADisplayLink *displayLink;
@end

@implementation SSSViewController 

@synthesize animating, context, displayLink;
@synthesize nameLabel, descriptionLabel;
@synthesize nameTextField, descriptionTextField;
@synthesize audioRecList, noteRecList, rhythmList, noteTrackList, audioTrackList, setList;

////////----- GUI FUNCTIONS ---------////////
bool g_uiAlpha;


- (IBAction) toggleUI:(UISwitch *)sender{
    g_uiAlpha = !g_uiAlpha; 
    
    // assign to the IBOutlest:
    ActiveSwitchOutlet.alpha = g_uiAlpha;
    ActiveSwitchOutlet.alpha = g_uiAlpha;
    
    nameTextOutlet.alpha = g_uiAlpha;
    DescriptionTextOutlet.alpha = g_uiAlpha;
    uploadButtonOutlet.alpha = g_uiAlpha;
    downloadButtonOutlet.alpha = g_uiAlpha;
    
    nameLabel.alpha = g_uiAlpha;
    descriptionLabel.alpha = g_uiAlpha;
    
    uploadSoundButtonOutlet.alpha = g_uiAlpha;
    downloadSoundButtonOutlet.alpha = g_uiAlpha;
    
    uploadRhythmButtonOutlet.alpha = g_uiAlpha;
    downloadRhythmButtonOutlet.alpha = g_uiAlpha;
    
    uploadTrackButtonOutlet.alpha = g_uiAlpha;
    downloadTrackButtonOutlet.alpha = g_uiAlpha;
    
    uploadButtonOutlet.alpha = g_uiAlpha;
    downloadButtonOutlet.alpha = g_uiAlpha;
    
    //micLabel.alpha = g_uiAlpha;
}

- (IBAction) ClearButton:(UISwitch *)sender{
    Erase();
}

- (IBAction) StopButton:(UISwitch *)sender{
    Stop();
}

- (IBAction) PlayButton:(UISwitch *)sender{
    Play();
}

- (IBAction) RecordButton:(UISwitch *)sender{
    Record();
}


- (IBAction)DubButton:(UIButton *)sender{
    Dub();
}


- (IBAction)downloadSound:(UIButton *)sender
{
    if (getCurrentTrack() == AUDIO_TRACK || getCurrentTrack() == AUDIO_TRACK_2){
        g_loadType = AUDIO_REC_ONLY;
        modalTest.sounds = audioRecList;
        //// modalTest.sounds = [[NSMutableArray alloc] initWithObjects:audioRecList, nil]; 
        //modalTest.sounds = [[NSMutableArray alloc] init];
        //[modalTest.sounds addObjectsFromArray:audioRecList];
    }
    else if (getCurrentTrack() == BASS_TRACK || getCurrentTrack() ==LEAD_TRACK || getCurrentTrack() == CHORD_TRACK){
        g_loadType = NOTE_REC_ONLY;
        modalTest.sounds = noteRecList;
    }
    
    [modalTest.soundTableView reloadData]; 

    [self resignFirstResponder];
    [self presentModalViewController:modalTest animated:YES];
}

- (IBAction)downloadRhythm:(UIButton *)sender
{
    modalTest.sounds = rhythmList;
    
    [modalTest.soundTableView reloadData]; 
    
    g_loadType = RHYTHM_ONLY;
    [self resignFirstResponder];
    [self presentModalViewController:modalTest animated:YES];
}

- (IBAction)downloadTrack:(UIButton *)sender
{
    if (getCurrentTrack() == AUDIO_TRACK || getCurrentTrack() == AUDIO_TRACK_2){
        g_loadType = AUDIO_PLUS_RHYTHM;
        modalTest.sounds = audioTrackList;
    }
    else if (getCurrentTrack() == BASS_TRACK || getCurrentTrack() ==LEAD_TRACK || getCurrentTrack() == CHORD_TRACK)
    {
        g_loadType = NOTE_PLUS_RHYTHM;
        modalTest.sounds = noteTrackList;
    }
        
    [modalTest.soundTableView reloadData]; 
    
    [self resignFirstResponder];
    [self presentModalViewController:modalTest animated:YES];
}


- (IBAction)downloadSet:(UIButton *)sender
{
    modalTest.sounds = setList;
    
    [modalTest.soundTableView reloadData]; 
    
    g_loadType = SONG_SET;
    [self resignFirstResponder];
    [self presentModalViewController:modalTest animated:YES];
}


- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    touchesBegan(touches);
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    touchesMoved(touches);
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    touchesEnded(touches);
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
    [self touchesEnded:touches withEvent:event];
}

////////----- GUI FUNCTIONS ---------////////

//// networking
- (IBAction)uploadSound:(id)sender
{
    // fill basic fields:
    [nameTextField resignFirstResponder]; 
    [descriptionTextField resignFirstResponder];
    // UPLOAD A FILE
    NSMutableDictionary *parameters = [NSMutableDictionary dictionary];
    [parameters setObject:nameTextField.text forKey:@"name"];
    [parameters setObject:descriptionTextField.text forKey:@"description"];
    CLLocation *location = [locationManager location];
    CLLocationCoordinate2D coordinate = location.coordinate;
    CLLocationDegrees latitude = coordinate.latitude;
    CLLocationDegrees longitude = coordinate.longitude;
    [parameters setObject:[NSNumber numberWithFloat:latitude] forKey:@"lat"];
    [parameters setObject:[NSNumber numberWithFloat:longitude] forKey:@"long"];
    [parameters setObject:[[UIDevice currentDevice] uniqueIdentifier] forKey:@"udid"];
    
    NSMutableURLRequest *myRequest;
    AFHTTPRequestOperation *uploadOperation;
    NSData *recData;
    
    if(getCurrentTrack() == AUDIO_TRACK || getCurrentTrack() == AUDIO_TRACK_2){
        audioRecording aRecording;
        aRecording = getAudioRecording();
        // Make NSDat object:
        recData = [NSData dataWithBytes:aRecording.audio_track length:(aRecording.length) * sizeof(SAMPLE)];
        
        myRequest = [[SongShareClient sharedClient] multipartFormRequestWithMethod:@"POST" path:@"songshare/sound" parameters:parameters constructingBodyWithBlock: ^(id <AFMultipartFormData>formData) {
        [formData appendPartWithFileData:recData name:@"soundfile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"]; 
        }];
    }
    else if(getCurrentTrack() == CHORD_TRACK || getCurrentTrack() == BASS_TRACK || getCurrentTrack() == LEAD_TRACK){
        synthRecording recording;
        recording = getNoteRecording();
        // Make NSDat object:
        recData = [NSData dataWithBytes:recording.note_data length:(recording.length) * sizeof(recordData)];
        
        myRequest = [[SongShareClient sharedClient] multipartFormRequestWithMethod:@"POST" path:@"songshare/note" parameters:parameters constructingBodyWithBlock: ^(id <AFMultipartFormData>formData) {
            [formData appendPartWithFileData:recData name:@"notefile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"]; 
        }];
    }
    
    uploadOperation = [[AFHTTPRequestOperation alloc] initWithRequest:myRequest];
    
    [uploadOperation setCompletionBlockWithSuccess:^(AFHTTPRequestOperation *operation, id responseObject) {
        [self refresh]; // changed from weakSelf
    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        NSLog(@"FAILURE TO UPLOAD: %@", error);
    }];
    
    
    NSOperationQueue *queue = [NSOperationQueue mainQueue];
    [queue addOperation:uploadOperation];
}

- (IBAction)uploadRhythm:(id)sender
{
    // fill basic fields:
    [nameTextField resignFirstResponder]; 
    [descriptionTextField resignFirstResponder];
    // UPLOAD A FILE
    NSMutableDictionary *parameters = [NSMutableDictionary dictionary];
    [parameters setObject:nameTextField.text forKey:@"name"];
    [parameters setObject:descriptionTextField.text forKey:@"description"];
    CLLocation *location = [locationManager location];
    CLLocationCoordinate2D coordinate = location.coordinate;
    CLLocationDegrees latitude = coordinate.latitude;
    CLLocationDegrees longitude = coordinate.longitude;
    [parameters setObject:[NSNumber numberWithFloat:latitude] forKey:@"lat"];
    [parameters setObject:[NSNumber numberWithFloat:longitude] forKey:@"long"];
    [parameters setObject:[[UIDevice currentDevice] uniqueIdentifier] forKey:@"udid"];
    
    NSMutableURLRequest *myRequest;
    AFHTTPRequestOperation *uploadOperation;
    NSData *recData;
    
    artRecording recording;
    recording = getArtRecording();
    // Make NSDat object:
    recData = [NSData dataWithBytes:recording.art_data length:(recording.length) * sizeof(recordData)];
    
    myRequest = [[SongShareClient sharedClient] multipartFormRequestWithMethod:@"POST" path:@"songshare/rhythm" parameters:parameters constructingBodyWithBlock: ^(id <AFMultipartFormData>formData) {
            [formData appendPartWithFileData:recData name:@"rhythmfile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"]; 
    }];
    
    uploadOperation = [[AFHTTPRequestOperation alloc] initWithRequest:myRequest];
    
    [uploadOperation setCompletionBlockWithSuccess:^(AFHTTPRequestOperation *operation, id responseObject) {
        [self refresh]; 
    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        NSLog(@"FAILURE TO UPLOAD: %@", error);
    }];
    
    NSOperationQueue *queue = [NSOperationQueue mainQueue];
    [queue addOperation:uploadOperation];
    
    
    
}

- (IBAction)uploadTrack:(id)sender
{
    // fill basic fields:
    [nameTextField resignFirstResponder]; // would like to append something like:rhythm for track, notes for track->
    [descriptionTextField resignFirstResponder];
    // UPLOAD Rhythm file:
    NSMutableDictionary *parameters = [NSMutableDictionary dictionary];
    [parameters setObject:nameTextField.text forKey:@"name"];
    [parameters setObject:descriptionTextField.text forKey:@"description"];
    CLLocation *location = [locationManager location];
    CLLocationCoordinate2D coordinate = location.coordinate;
    CLLocationDegrees latitude = coordinate.latitude;
    CLLocationDegrees longitude = coordinate.longitude;
    [parameters setObject:[NSNumber numberWithFloat:latitude] forKey:@"lat"];
    [parameters setObject:[NSNumber numberWithFloat:longitude] forKey:@"long"];
    [parameters setObject:[[UIDevice currentDevice] uniqueIdentifier] forKey:@"udid"];
    
    NSMutableURLRequest *myRequest;
    AFHTTPRequestOperation *uploadOperation;
    NSData *artData;
    NSData *soundData;
    
    ///////// - Rhythm track upload - ////////////
    // ---------------------------------------- //
    
    artRecording recording;
    recording = getArtRecording();
    // Make NSDat object:
    artData = [NSData dataWithBytes:recording.art_data length:(recording.length) * sizeof(recordData)];
    
    ///// - Notes or Audio upload + rhythm - //////
    /////
    if(getCurrentTrack() == AUDIO_TRACK || getCurrentTrack() == AUDIO_TRACK_2){
        audioRecording aRecording;
        aRecording = getAudioRecording();
        // Make NSDat object:
        soundData = [NSData dataWithBytes:aRecording.audio_track length:(aRecording.length) * sizeof(SAMPLE)];
        
        myRequest = [[SongShareClient sharedClient] multipartFormRequestWithMethod:@"POST" path:@"songshare/audiotrack" parameters:parameters constructingBodyWithBlock: ^(id <AFMultipartFormData>formData) {
            [formData appendPartWithFileData:soundData name:@"soundfile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"];
            [formData appendPartWithFileData:artData name:@"artfile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"]; 
        }];
    }
    else if(getCurrentTrack() == CHORD_TRACK || getCurrentTrack() == BASS_TRACK || getCurrentTrack() == LEAD_TRACK){
        synthRecording sRecording;
        sRecording = getNoteRecording();
        // Make NSDat object:
        soundData = [NSData dataWithBytes:sRecording.note_data length:(sRecording.length) * sizeof(recordData)];
        
        myRequest = [[SongShareClient sharedClient] multipartFormRequestWithMethod:@"POST" path:@"songshare/synthtrack" parameters:parameters constructingBodyWithBlock: ^(id <AFMultipartFormData>formData) {
            [formData appendPartWithFileData:soundData name:@"notefile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"]; 
            [formData appendPartWithFileData:artData name:@"artfile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"]; 
        }];
    }

    uploadOperation = [[AFHTTPRequestOperation alloc] initWithRequest:myRequest];
    NSLog(@"try %@", uploadOperation);
    [uploadOperation setCompletionBlockWithSuccess:^(AFHTTPRequestOperation *operation, id responseObject) {
        [self refresh]; 
    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        NSLog(@"FAILURE TO UPLOAD: %@", error);
    }];
    
    NSOperationQueue *queue = [NSOperationQueue mainQueue];
    [queue addOperation:uploadOperation];

}

////////
- (IBAction)uploadSet:(id)sender
{
    // fill basic fields:
    [nameTextField resignFirstResponder]; // would like to append something like:rhythm for track, notes for track->
    [descriptionTextField resignFirstResponder];
    // UPLOAD Rhythm file:
    NSMutableDictionary *parameters = [NSMutableDictionary dictionary];
    [parameters setObject:nameTextField.text forKey:@"name"];
    [parameters setObject:descriptionTextField.text forKey:@"description"];
    CLLocation *location = [locationManager location];
    CLLocationCoordinate2D coordinate = location.coordinate;
    CLLocationDegrees latitude = coordinate.latitude;
    CLLocationDegrees longitude = coordinate.longitude;
    [parameters setObject:[NSNumber numberWithFloat:latitude] forKey:@"lat"];
    [parameters setObject:[NSNumber numberWithFloat:longitude] forKey:@"long"];
    [parameters setObject:[[UIDevice currentDevice] uniqueIdentifier] forKey:@"udid"];
    
    NSMutableURLRequest *myRequest;
    AFHTTPRequestOperation *uploadOperation;

    // get note data
    synthRecording chordTrack, bassTrack, leadTrack;
    chordTrack = getNoteRecording(CHORD_TRACK);
    bassTrack = getNoteRecording(BASS_TRACK);
    leadTrack = getNoteRecording(LEAD_TRACK);
    
    // get audio data
    audioRecording audioTrack1, audioTrack2;
    audioTrack1 = getAudioRecording(AUDIO_TRACK);
    audioTrack2 = getAudioRecording(AUDIO_TRACK_2);

    // get rhythm data
    artRecording chordArt, bassArt, leadArt, audio1Art, audio2Art;
    chordArt = getArtRecording(CHORD_TRACK);
    bassArt = getArtRecording(BASS_TRACK);
    leadArt = getArtRecording(LEAD_TRACK);
    audio1Art = getArtRecording(AUDIO_TRACK);
    audio2Art = getArtRecording(AUDIO_TRACK_2);


    // fill NSDat objects:
    // synth and audio track data:
    NSData * chordData = [NSData dataWithBytes:chordTrack.note_data length:(chordTrack.length) * sizeof(recordData)];
    NSData * bassData = [NSData dataWithBytes:bassTrack.note_data length:(bassTrack.length) * sizeof(recordData)];
    NSData * leadData = [NSData dataWithBytes:leadTrack.note_data length:(leadTrack.length) * sizeof(recordData)];
    NSData * audio1Data = [NSData dataWithBytes:audioTrack1.audio_track length:(audioTrack1.length) * sizeof(SAMPLE)];
    NSData * audio2Data = [NSData dataWithBytes:audioTrack2.audio_track length:(audioTrack2.length) * sizeof(SAMPLE)];
    // rhythm / articulation data:
    NSData * artChordData = [NSData dataWithBytes:chordArt.art_data length:(chordArt.length) * sizeof(recordData)];
    NSData * artBassData = [NSData dataWithBytes:bassArt.art_data length:(bassArt.length) * sizeof(recordData)];
    NSData * artLeadData = [NSData dataWithBytes:leadArt.art_data length:(leadArt.length) * sizeof(recordData)];
    NSData * artAudio1Data = [NSData dataWithBytes:audio1Art.art_data length:(audio1Art.length) * sizeof(recordData)];
    NSData * artAudio2Data = [NSData dataWithBytes:audio1Art.art_data length:(audio2Art.length) * sizeof(recordData)];

    ////

        
        myRequest = [[SongShareClient sharedClient] multipartFormRequestWithMethod:@"POST" path:@"songshare/band" parameters:parameters constructingBodyWithBlock: ^(id <AFMultipartFormData>formData) {
            [formData appendPartWithFileData:chordData name:@"chordfile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"];
            [formData appendPartWithFileData:bassData name:@"bassfile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"]; 
            [formData appendPartWithFileData:leadData name:@"leadfile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"];
            [formData appendPartWithFileData:audio1Data name:@"audio1file" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"]; 
            [formData appendPartWithFileData:audio2Data name:@"audio2file" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"];
            // rhythm data
            [formData appendPartWithFileData:artChordData name:@"artchordfile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"];
            [formData appendPartWithFileData:artBassData name:@"artbassfile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"]; 
            [formData appendPartWithFileData:artLeadData name:@"artleadfile" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"];
            [formData appendPartWithFileData:artAudio1Data name:@"artaudio1file" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"]; 
            [formData appendPartWithFileData:artAudio2Data name:@"artaudio2file" fileName:[NSString mf_stringWithUUID] mimeType: @"audio/wav"];
        }];


    
    uploadOperation = [[AFHTTPRequestOperation alloc] initWithRequest:myRequest];   
    [uploadOperation setCompletionBlockWithSuccess:^(AFHTTPRequestOperation *operation, id responseObject) {
        [self refresh]; 
    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        NSLog(@"FAILURE TO UPLOAD: %@", error);
    }];
    
    NSOperationQueue *queue = [NSOperationQueue mainQueue];
    [queue addOperation:uploadOperation];
}

// ----------------------------------
// refresh()
// refreshes the list of things to display from the server
// ----------------------------------
- (void )refresh
{
    NSLog(@"REFRESHING");
    [[SongShareClient sharedClient] getPath:@"songshare/sounds" parameters:nil success:^(__unused AFHTTPRequestOperation *operation, id JSON) {
        NSLog(@"GOT JSON: %@", JSON);
        
        NSMutableArray *audio_mutableRecords = [NSMutableArray array];
        for (NSDictionary *sound in JSON) {
            [audio_mutableRecords addObject:sound];
        }

        audioRecList = audio_mutableRecords;
         [modalTest.soundTableView reloadData]; 
        
    } failure:^(__unused AFHTTPRequestOperation *operation, NSError *error) {
        NSLog(@"FAILURE, %@", error);
    }];
    
    ///// For note recording
    [[SongShareClient sharedClient] getPath:@"songshare/notes" parameters:nil success:^(__unused AFHTTPRequestOperation *operation, id JSON) {
        
        NSMutableArray *note_mutableRecords = [NSMutableArray array];
        for (NSDictionary *note in JSON) {
            [note_mutableRecords addObject:note];
        }
        noteRecList = note_mutableRecords;
        
         [modalTest.soundTableView reloadData]; 
        
    } failure:^(__unused AFHTTPRequestOperation *operation, NSError *error) {
        NSLog(@"FAILURE, %@", error);
    }];
    
    ///// For rhythm recordings
    [[SongShareClient sharedClient] getPath:@"songshare/rhythms" parameters:nil success:^(__unused AFHTTPRequestOperation *operation, id JSON) {
        
        NSMutableArray *rhythm_mutableRecords = [NSMutableArray array];
        for (NSDictionary *note in JSON) {
            [rhythm_mutableRecords addObject:note];
        }
        rhythmList = rhythm_mutableRecords;
        
        [modalTest.soundTableView reloadData]; 
        
    } failure:^(__unused AFHTTPRequestOperation *operation, NSError *error) {
        NSLog(@"FAILURE, %@", error);
    }];
    
    ///// For note track
    [[SongShareClient sharedClient] getPath:@"songshare/synthtracks" parameters:nil success:^(__unused AFHTTPRequestOperation *operation, id JSON) {
        
        NSMutableArray *rhythm_mutableRecords = [NSMutableArray array];
        for (NSDictionary *note in JSON) {
            [rhythm_mutableRecords addObject:note];
        }
        noteTrackList = rhythm_mutableRecords;
        
        [modalTest.soundTableView reloadData]; 
        
    } failure:^(__unused AFHTTPRequestOperation *operation, NSError *error) {
        NSLog(@"FAILURE, %@", error);
    }];

    
    ///// For audio track
    [[SongShareClient sharedClient] getPath:@"songshare/audiotracks" parameters:nil success:^(__unused AFHTTPRequestOperation *operation, id JSON) {
        
        NSMutableArray *rhythm_mutableRecords = [NSMutableArray array];
        for (NSDictionary *note in JSON) {
            [rhythm_mutableRecords addObject:note];
        }
        audioTrackList = rhythm_mutableRecords;
        
        [modalTest.soundTableView reloadData]; 
        
    } failure:^(__unused AFHTTPRequestOperation *operation, NSError *error) {
        NSLog(@"FAILURE, %@", error);
    }];

    
    ///// For complete set
    [[SongShareClient sharedClient] getPath:@"songshare/bands" parameters:nil success:^(__unused AFHTTPRequestOperation *operation, id JSON) {
        
        NSMutableArray *rhythm_mutableRecords = [NSMutableArray array];
        for (NSDictionary *note in JSON) {
            [rhythm_mutableRecords addObject:note];
        }
        setList  = rhythm_mutableRecords;
        
        [modalTest.soundTableView reloadData]; 
        
    } failure:^(__unused AFHTTPRequestOperation *operation, NSError *error) {
        NSLog(@"FAILURE, %@", error);
    }];

} 

// ----------------------------------
// loadRecording()
// Takes an audio recording track (presumably from the server) and loads it into the current track
-(void) loadRecording:(SAMPLE *)data :(int)numFrames{
    if (g_loadType == AUDIO_REC_ONLY || g_loadType == AUDIO_PLUS_RHYTHM){
        // pack into struct
        audioRecording recording;
        recording.audio_track = data;
        recording.length = numFrames;
        
        // load it into the current track:
        loadAudioRecording(recording);
    }
}
-(void) loadNoteData:(recordData *)noteData :(int)numFrames{
    if (g_loadType == NOTE_REC_ONLY || g_loadType == NOTE_PLUS_RHYTHM){
        // pack into struct
        synthRecording recording;
        recording.note_data = noteData;
        recording.length = numFrames;
        
        // load it into the current track:
        loadNoteRecording(recording);
    }
}

-(void) loadRhythmData:(recordData *)rhythmData :(int)numFrames{
    if (g_loadType == RHYTHM_ONLY || g_loadType == NOTE_PLUS_RHYTHM || g_loadType == AUDIO_PLUS_RHYTHM){
        // pack into struct
        artRecording recording;
        recording.art_data = rhythmData;
        recording.length = numFrames;
        
        // load it into the current track:
        loadArtRecording(recording);
    }
}

-(void) loadSet:(recordData *)chordData :(int)numChordData :(recordData *)bassData :(int)numBassData :(recordData *)leadData :(int)numLeadData :(SAMPLE *)audio1Data :(int)numAudio1Data :(SAMPLE *)audio2Data :(int)numAudio2Data :(recordData *)artChordData :(int)numArtChordData :(recordData *)artBassData :(int)numArtBassData :(recordData *)artLeadData :(int)numArtLeadData :(recordData *)artAudio1Data :(int)numArtAudio1Data :(recordData *)artAudio2Data :(int)numArtAudio2Data 
{
    synthRecording chordRecording, bassRecording, leadRecording;
    // Chord Notes
    chordRecording.note_data = chordData;
    chordRecording.length = numChordData;
    loadNoteRecording(CHORD_TRACK, chordRecording);
    // Bass Notes
    bassRecording.note_data = bassData;
    bassRecording.length = numBassData;
    loadNoteRecording(BASS_TRACK, bassRecording);
    // Lead Notes
    leadRecording.note_data = leadData;
    leadRecording.length = numLeadData;
    loadNoteRecording(LEAD_TRACK, leadRecording);
    // load audio track 1
    audioRecording audio1, audio2;
    audio1.audio_track = audio1Data;
    audio1.length = numAudio1Data;
    loadAudioRecording(AUDIO_TRACK, audio1);
    // audio track 2
    audio2.audio_track = audio2Data;
    audio2.length = numAudio2Data;
    loadAudioRecording(AUDIO_TRACK_2, audio2);
    
    // Rhythm
    artRecording artChord, artBass, artLead, artAudio1, artAudio2;
    artChord.art_data = artChordData;
    artChord.length = numArtChordData;
    loadArtRecording(CHORD_TRACK, artChord);
    // bass
    artBass.art_data = artBassData;
    artBass.length = numArtBassData;
    loadArtRecording(BASS_TRACK, artBass);
    // Lead
    artLead.art_data = artLeadData;
    artLead.length = numArtLeadData;
    loadArtRecording(LEAD_TRACK, artLead);
    // Audio 1
    artAudio1.art_data = artAudio1Data;
    artAudio1.length = numArtAudio1Data;
    loadArtRecording(AUDIO_TRACK, artAudio1);
    // Audio 2
    artAudio2.art_data = artAudio2Data;
    artAudio2.length = numArtAudio2Data;
    loadArtRecording(AUDIO_TRACK_2, artAudio2);
}

-(int) getLoadType{
    return g_loadType;
}


/// \ networking

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if ( self )
    {
        EAGLContext * aContext = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
        
        if (!aContext)
            NSLog(@"Failed to create ES context");
        else if (![EAGLContext setCurrentContext:aContext])
            NSLog(@"Failed to set ES context current");
        
        self.context = aContext;
        
        [(EAGLView *)self.view setContext:context];
        [(EAGLView *)self.view setFramebuffer];
        
        animating = FALSE;
        animationFrameInterval = 1;
        self.displayLink = nil;
        
        
        nameLabel.textColor = [UIColor whiteColor];
        descriptionLabel.textColor = [UIColor whiteColor];
        // init alpha
        nameLabel.alpha = g_uiAlpha;
        descriptionLabel.alpha = g_uiAlpha;
        // buttons
        ActiveSwitchOutlet.alpha = g_uiAlpha;
        ActiveSwitchOutlet.alpha = g_uiAlpha;
        nameTextOutlet.alpha = g_uiAlpha;
        DescriptionTextOutlet.alpha = g_uiAlpha;
        uploadButtonOutlet.alpha = g_uiAlpha;
        downloadButtonOutlet.alpha = g_uiAlpha;
        
        uploadSoundButtonOutlet.alpha = g_uiAlpha;
        downloadSoundButtonOutlet.alpha = g_uiAlpha;
        
        uploadRhythmButtonOutlet.alpha = g_uiAlpha;
        downloadRhythmButtonOutlet.alpha = g_uiAlpha;
        
        uploadTrackButtonOutlet.alpha = g_uiAlpha;
        downloadTrackButtonOutlet.alpha = g_uiAlpha;
    }
    return self;
}


- (void)dealloc
{
    if (program)
    {
        glDeleteProgram(program);
        program = 0;
    }
    
    // Tear down context.
    if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
}

- (void)viewWillAppear:(BOOL)animated
{
    [self startAnimation];
    
    [super viewWillAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [self stopAnimation];
    
    [super viewWillDisappear:animated];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    
    locationManager = [[CLLocationManager alloc] init];
    [locationManager startUpdatingLocation];
    
    // instantiate
    theClient = [SongShareClient sharedClient];
    
    modalTest = [[ModalTest alloc] initWithNibName:@"ModalTest" bundle:[NSBundle mainBundle]];
    modalTest.mainView = self;
    
    [self refresh];
}

- (void)viewDidUnload
{
	[super viewDidUnload];
	
    if (program)
    {
        glDeleteProgram(program);
        program = 0;
    }
    
    // Tear down context.
    if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
	self.context = nil;	
}

- (NSInteger)animationFrameInterval
{
    return animationFrameInterval;
}

- (void)setAnimationFrameInterval:(NSInteger)frameInterval
{
    /*
	 Frame interval defines how many display frames must pass between each time the display link fires.
	 The display link will only fire 30 times a second when the frame internal is two on a display that refreshes 60 times a second. The default frame interval setting of one will fire 60 times a second when the display refreshes at 60 times a second. A frame interval setting of less than one results in undefined behavior.
	 */
    if (frameInterval >= 1)
    {
        animationFrameInterval = frameInterval;
        
        if (animating)
        {
            [self stopAnimation];
            [self startAnimation];
        }
    }
}

- (void)startAnimation
{
    if (!animating)
    {
        CADisplayLink *aDisplayLink = [CADisplayLink displayLinkWithTarget:self selector:@selector(drawFrame)];
        [aDisplayLink setFrameInterval:animationFrameInterval];
        [aDisplayLink addToRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
        self.displayLink = aDisplayLink;
        
        animating = TRUE;
    }
}

- (void)stopAnimation
{
    if (animating)
    {
        [self.displayLink invalidate];
        self.displayLink = nil;
        animating = FALSE;
    }
}

- (void)drawFrame
{
    [(EAGLView *)self.view setFramebuffer];
    
    // ge: render
    yayRender();
    
    [(EAGLView *)self.view presentFramebuffer];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc. that aren't in use.
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    // change to NO to force it stay in one mode
    return NO;//(interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

@end
