//-----------------------------------------------------------------------------
//  name: renderer.mm
//  name: graphics + audio for HellOGLES
//
//  author: Ge Wang
//  date: winter 2011
//-----------------------------------------------------------------------------
#import "renderer.h"
#import "mo_audio.h"
#import "mo_touch.h"
#import "mo_gfx.h"
#import "JifDefines.h"

// global
SAMPLE g_width = 1.5;
GLint g_GfxWidth = 320;
GLint g_GfxHeight = 480;

// buffer
SAMPLE g_vertices[FRAMESIZE*4]; // made to be 4 times requested frame size since it is non-deterministic
// last frame size
UInt32 g_lastNumFrames = 0;

// touch entities!
TouchEntity * g_touches[MAX_TOUCHES];
// number of active touches
UInt32 g_numActiveTouches = 0;

//background hexes
int g_hex_radius=40;
PulseEntity ** g_hexes;
// number of background hexes
int g_num_hexes;

// Squares
SquareEntity ** g_squares;
int g_numSquares;

MixMaster * g_mixer;
int g_currentTrack;
int g_playMode;
bool g_trackUpdated;
CGPoint g_initTouchPos;

// Colors
Vector3D g_b6=Vector3D(0.1f,0.1f,.5f);
Vector3D g_b5=Vector3D(0.2f,0.2f,.6f);
Vector3D g_b4=Vector3D(0.3f,0.3f,.7f);
Vector3D g_b3=Vector3D(0.4f,0.4f,.8f);
Vector3D g_b2=Vector3D(0.5f,0.5f,.9f);
Vector3D g_b1=Vector3D(0.6f,0.6f,1.0f);
Vector3D g_b0=Vector3D(0.7f,0.7f,1.0f);

Vector3D g_r6=Vector3D(0.6f,0.1f,0.2f);
Vector3D g_r5=Vector3D(0.65f,0.1f,0.2f);
Vector3D g_r4=Vector3D(0.7f,0.1f,0.2f);
Vector3D g_r3=Vector3D(0.75f,0.1f,0.2f);
Vector3D g_r2=Vector3D(0.8f,0.1f,0.2f);
Vector3D g_r1=Vector3D(0.85f,0.1f,0.2f);
Vector3D g_r0=Vector3D(0.9f,0.1f,0.2f);


//-----------------------------------------------------------------------------
// name: load Chordy()
// - set up colors and mapping for chords
//-----------------------------------------------------------------------------
void initChordy(){
    g_currentTrack = CHORD_TRACK;
    //  Hexagons
    // instantiate background hex objects
    int hex_wide=int(ceil((g_GfxWidth/(2*g_hex_radius/sqrt(3)))));
    int hex_height=int(ceil(g_GfxHeight/((2*g_hex_radius)/sqrt(3))));
    
    g_num_hexes=hex_wide*hex_height; // minus one width
    g_hexes=new PulseEntity*[hex_wide*hex_height];
    
    int MAJOR_DIATONIC[7] = {ONE_MAJ, TWO_MAJ, THREE_MAJ, FOUR_MAJ, FIVE_MAJ, SIX_MAJ, SEVEN_MAJ};
    Vector3D COLOR_WHEEL[7] = {g_b0, g_b1, g_b2, g_b3, g_b4, g_b5, g_b6};
    
    for (int i=0;i<hex_wide;++i){
        for (int j=0; j<hex_height;++j){
            g_hexes[i*hex_height+j]=new PulseEntity();
            PulseEntity * cur= g_hexes[i*hex_height+j];
            cur->num_sides=6;
            cur->radius=g_hex_radius;
            
            int index;
            int offset = 1; // use this to shift the root et. al and keep relationship
            if (j%2==0){
                cur->loc=Vector3D((3*i+2)*g_hex_radius,float(j)*g_hex_radius*sqrt(3)/2.0f,0);
                
                
                index =  (int)((( 4 * i +  (j + 2 ) ) + offset) % 7) ;
                cur->setScaleDegree(MAJOR_DIATONIC[ index]); // same row... increment in 5ths
                cur->set_color(COLOR_WHEEL[index]);
                //NSLog(@"i: %i, j: %i, index evn: %i",i, j, MAJOR_DIATONIC[ index]);
            }
            else{
                cur->loc=Vector3D(3*i*g_hex_radius+(g_hex_radius/2.0f),float(j)*g_hex_radius*sqrt(3)/2.0f,0);
                index = (int)((( 4 * i + j  ) + offset) % 7) ;
                cur->setScaleDegree(MAJOR_DIATONIC[index]);
                cur->set_color(COLOR_WHEEL[index]);
                //NSLog(@"i: %i, j: %i, index odd: %i",i, j, MAJOR_DIATONIC[ index]);
            }     
            
        }
    }
}

void initSquares(){
    float sq_width = g_GfxHeight/9;
    Vector3D location;
    int wide = 8;
    int tall = 5;
    g_numSquares = wide * tall;
    
    g_squares = new SquareEntity*[g_numSquares];
    for (int i = 0; i < g_numSquares; i++){
        g_squares[i] = new SquareEntity();
    }
    
    Vector3D COLOR_WHEEL[7] = {g_b0, g_b1, g_b2, g_b3, g_b4, g_b5, g_b6};
    
    for (int j = 0; j < tall; j++){
        for (int i = 0; i < wide; i++){
            int index = i + wide*j;
            g_squares[index]->m_width = sq_width;
            
            location.x = i * sq_width + sq_width/2;
            location.y = j * sq_width + sq_width/2;
            
            //NSLog(@"index: %i, x %f, y: %f", index, location.x, location.y);
            
            g_squares[index]->loc = location;
            g_squares[index]->m_keyLabel = i + 2*j;
            g_squares[index]->col = COLOR_WHEEL[g_squares[index]->m_keyLabel%7];
        }
    }
}

void setSquareColor(){
    int wide = 8;
    int tall = 5;

    Vector3D BLUE_COLOR_WHEEL[7] = {g_b0, g_b1, g_b2, g_b3, g_b4, g_b5, g_b6};
    Vector3D RED_COLOR_WHEEL[7] = {g_r0, g_r1, g_r2, g_r3, g_r4, g_r5, g_r6};
    
    for (int j = 0; j < tall; j++){
        for (int i = 0; i < wide; i++){
            int index = i + wide*j;
            if(g_currentTrack == BASS_TRACK)
                g_squares[index]->col = BLUE_COLOR_WHEEL[g_squares[index]->m_keyLabel%7];
            else if(g_currentTrack == LEAD_TRACK)
                g_squares[index]->col = RED_COLOR_WHEEL[g_squares[index]->m_keyLabel%7];
        }
    }
}

//-----------------------------------------------------------------------------
// name: audio_callback()
// desc: audio callback, yeah
//-----------------------------------------------------------------------------
void audio_callback( Float32 * buffer, UInt32 numFrames, void * userData )
{
    g_lastNumFrames = numFrames;

    // recorder
    g_mixer->tick(buffer, numFrames);
    
    // HACK HACK HACK
    if (g_numActiveTouches > 0 && getCurrentNote() != g_mixer->getKeyName()){
        
        if (g_currentTrack != CHORD_TRACK ){
            int note = getCurrentNote();
            
            // send message to the mixMaster
            if (note != -1 && g_playMode != RECORD_ARTICULATION){
                if (g_currentTrack ==LEAD_TRACK || g_currentTrack == BASS_TRACK)
                    g_mixer->setNote(note);
                
            }
        }
        else if (g_currentTrack == CHORD_TRACK && g_playMode != RECORD_ARTICULATION)
        {
            PulseEntity * currentTouch = find_hex(g_touches[0]->loc);
            int chord = currentTouch->m_accompaniment->getScaleDegree();
            //NSLog(@"setting chord: %i", chord);
            g_mixer->setNote(chord);
        }
    }
}

void touchesBegan(NSSet *touches)
{
    CGPoint location;
    for( UITouch * touch in touches )
    {
        // get the location
        location = [touch locationInView:nil];
        
        // transform: to make landscape
        double temp = location.x;
        location.x = location.y;
        location.y = temp;
        
        // find idle touch entity
        TouchEntity * entity = NULL;
        for( int i = 0; i < MAX_TOUCHES; i++ )
        {
            // in case touch already active
            if( g_touches[i]->touch_ref == touch )
                break;
            
            // find the next non-active touch
            if( !g_touches[i]->active )
            {
                entity = g_touches[i];
                break;
            }
        }
        
        // sanity check
        if( entity != NULL )
        {
            // set
            entity->active = true;
            entity->touch_ref = touch;
            entity->loc.x = location.x;
            entity->loc.y = location.y;                
            // count
            g_numActiveTouches++;
            // log it
            //NSLog( @"active touches: %d", g_numActiveTouches );
        }
        
        // send keyOn if one touch
        if (g_numActiveTouches == 1){
            g_trackUpdated = false;
            g_mixer->keyOn();
            g_initTouchPos.x = g_touches[0]->loc.x;
            g_initTouchPos.y = g_touches[0]->loc.y;
        }
    }
}

void touchesMoved(NSSet *touches)
{
    CGPoint location;
    for( UITouch * touch in touches )
    {
        // get the location
        location = [touch locationInView:nil];
        
        // transform: to make landscape
        double temp = location.x;
        location.x = location.y;
        location.y = temp;
        
        for( int i = 0; i < MAX_TOUCHES; i++ )
        {
            if( g_touches[i]->touch_ref == touch )
            {
                g_touches[i]->loc.x = location.x;
                g_touches[i]->loc.y = location.y;
                break;
            }
        }
        // 2 finger swipe to change instruments
        if (g_numActiveTouches == 2 && !g_trackUpdated){
            int difference = g_touches[0]->loc.x - g_initTouchPos.x;
            if (difference > 30){
                // before changing insts, key off
                g_mixer->keyOff();
                g_currentTrack++;
                g_currentTrack %= (NUM_TRACKS-1);
                g_trackUpdated = true;
                //                    NSLog(@"current Track + : %i", g_currentTrack);
            }
            if (difference < -30){
                g_mixer->keyOff();
                g_currentTrack--;
                if (g_currentTrack < 0)
                    g_currentTrack += (NUM_TRACKS-1);
                g_trackUpdated = true;
                //                    NSLog(@"current Track - : %i", g_currentTrack);
            }
            
        }
    }
}

void touchesEnded(NSSet *touches)
{
    CGPoint location;
    for( UITouch * touch in touches )
    {
        // get the location
        location = [touch locationInView:nil];
        
        // transform: to make landscape
        double temp = location.x;
        location.x = location.y;
        location.y = temp;
        
        for( int i = 0; i < MAX_TOUCHES; i++ )
        {
            if( g_touches[i]->touch_ref == touch )
            {
                // set
                g_touches[i]->active = false;
                g_touches[i]->touch_ref = NULL;
                
                // pack active touches
                for( int j = i+1; j < MAX_TOUCHES; j++ )
                {
                    if( g_touches[j]->active )
                    {
                        // swap
                        TouchEntity * swap = g_touches[i];
                        g_touches[i] = g_touches[j];
                        g_touches[j] = swap;
                        // dangerous: advance i
                        i = j;
                    }
                }
                
                // count
                g_numActiveTouches--;
                // log
                //NSLog( @"active touches: %d", g_numActiveTouches );
                
                break;
            }
        }
        // when no more touches send keyOff message
        if (g_numActiveTouches < 1)
            g_mixer->keyOff();
    }
}

////-----------------------------------------------------------------------------
//// name: touch_callback()
//// desc: touch callback function
////-----------------------------------------------------------------------------
//void touch_callback( NSSet * touches, UIView * view, const std::vector<MoTouchTrack> & tracks, void * data)
//{
//    // iterate over touch points
//    CGPoint location;
//    for( UITouch * touch in touches )
//    {
//        // get the location
//        location = [touch locationInView:nil];
//        
//        // transform: to make landscape
//        double temp = location.x;
//        location.x = location.y;
//        location.y = temp;
//        
//        // NSLog( @"touch: %f, %f,", location.x, location.y );
//        
//        else if( touch.phase == UITouchPhaseMoved )
//        {
//            for( int i = 0; i < MAX_TOUCHES; i++ )
//            {
//                if( g_touches[i]->touch_ref == touch )
//                {
//                    g_touches[i]->loc.x = location.x;
//                    g_touches[i]->loc.y = location.y;
//                    break;
//                }
//            }
//            // 2 finger swipe to change instruments
//            if (g_numActiveTouches == 2 && !g_trackUpdated){
//                int difference = g_touches[0]->loc.x - g_initTouchPos.x;
//                if (difference > 30){
//                    // before changing insts, key off
//                    g_mixer->keyOff();
//                    g_currentTrack++;
//                    g_currentTrack %= (NUM_TRACKS-1);
//                    g_trackUpdated = true;
////                    NSLog(@"current Track + : %i", g_currentTrack);
//                }
//                if (difference < -30){
//                    g_mixer->keyOff();
//                    g_currentTrack--;
//                    if (g_currentTrack < 0)
//                        g_currentTrack += (NUM_TRACKS-1);
//                    g_trackUpdated = true;
////                    NSLog(@"current Track - : %i", g_currentTrack);
//                }
//                
//            }
//        }
//        else if( (touch.phase == UITouchPhaseEnded) || (touch.phase == UITouchPhaseCancelled) )
//        {
//            for( int i = 0; i < MAX_TOUCHES; i++ )
//            {
//                if( g_touches[i]->touch_ref == touch )
//                {
//                    // set
//                    g_touches[i]->active = false;
//                    g_touches[i]->touch_ref = NULL;
//                    
//                    // pack active touches
//                    for( int j = i+1; j < MAX_TOUCHES; j++ )
//                    {
//                        if( g_touches[j]->active )
//                        {
//                            // swap
//                            TouchEntity * swap = g_touches[i];
//                            g_touches[i] = g_touches[j];
//                            g_touches[j] = swap;
//                            // dangerous: advance i
//                            i = j;
//                        }
//                    }
//                    
//                    // count
//                    g_numActiveTouches--;
//                    // log
//                    //NSLog( @"active touches: %d", g_numActiveTouches );
//                    
//                    break;
//                }
//            }
//         // when no more touches send keyOff message
//            if (g_numActiveTouches < 1)
//                g_mixer->keyOff();
//        }
//    }
//}


//-----------------------------------------------------------------------------
// name: setDims()
// desc: set dimension
//-----------------------------------------------------------------------------
void yaySetDims( GLint width, GLint height )
{
    g_GfxWidth = width;
    g_GfxHeight = height;
}



//-----------------------------------------------------------------------------
// name: init()
// desc: initialize graphics and sound
//-----------------------------------------------------------------------------
bool yayInit()
{
    
    /// init the mix master
    ///////////////////////
    g_mixer = new MixMaster();
    g_mixer->assignCurrentTrackPtr(&g_currentTrack);
    g_mixer->assignVertPtr(g_vertices);
    
    Stop();
    g_trackUpdated = false;
    g_playMode = STANDBY;
    
    // Chord Hexagons
    // start with chordal instrument ?
    initChordy();
    
    
    /// Init squares instrument
    //////////////////////
    initSquares();
    
    
    // instantiate
    for (int i = 0; i < MAX_TOUCHES; i++)
    {
        g_touches[i] = new TouchEntity();
        g_touches[i] ->col.set(1,1,1);
    }
    
//    MoTouch::addCallback(touch_callback, NULL);

    // init
    bool result = MoAudio::init( JIF_SRATE, FRAMESIZE, NUM_CHANNELS );
    if( !result )
    {
        // do not do this:
        int * p = 0;
        *p = 0;
    }
    // start
    result = MoAudio::start( audio_callback, NULL );
    if( !result )
    {
        // do not do this:
        int * p = 0;
        *p = 0;
    }
	
	return true;
}




//-----------------------------------------------------------------------------
// name: cleanup()
// desc: clean up
//-----------------------------------------------------------------------------
void yayCleanup()
{}




//-----------------------------------------------------------------------------
// name: render()
// desc: render next frame of graphics
//-----------------------------------------------------------------------------
void yayRender()
{
    // set to ortho
    MoGfx::ortho( g_GfxWidth, g_GfxHeight, 1 );
    
    // set clear color
    glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
    // clear it
    glClear( GL_COLOR_BUFFER_BIT );

    if (g_currentTrack == AUDIO_TRACK || g_currentTrack == AUDIO_TRACK_2)
        drawWaveForm();
    if (g_currentTrack == LEAD_TRACK || g_currentTrack == BASS_TRACK)
        drawInst();
    if (g_currentTrack == CHORD_TRACK)
        drawHexBackground();
}

//-----------------------------------------------------------------------------
// name: drawwaveForm()
// desc: draw Wave Form of incomin buffer
//-----------------------------------------------------------------------------
void drawWaveForm()
{
    MoGfx::perspective(90, g_GfxWidth/g_GfxHeight, 0.0, 100);
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    // "portait mode"
    glOrthof(-1.0, 1.0f, -1.5f, 1.5f, -1.0f, 1.0f);
    glLoadIdentity();
    
    if(g_currentTrack == AUDIO_TRACK)
        glClearColor(0.1f, 0.1f, 0.4f, .2f);
    else
        glClearColor(0.4f, 0.1f, 0.1f, 0.2f);
    
    glClear(GL_COLOR_BUFFER_BIT);
    
    // rotate around z
    glRotatef( 90, 0, 0, 1);
    
    // center
    glTranslatef(-g_width/2, 0, 0);
    
    glVertexPointer( 2, GL_FLOAT, 0, g_vertices);
    //enable it when you want to use it, then drawArrays
    glEnableClientState( GL_VERTEX_ARRAY);
    
    if (g_currentTrack == AUDIO_TRACK)
        glColor4f(0.8f, 0.3f, 0.3f, 1.0f);
    else
        glColor4f(0.3f, 0.3f, 0.9f, 1.0f);
    
    // g_lastNumFrames is how many to draw, if you do /2 it will just draw half the buffer
    // only have line_strip, loop? and triangle strip?
    glDrawArrays(GL_LINE_STRIP, 0, g_lastNumFrames);
    
    glDisableClientState(GL_VERTEX_ARRAY);
    
}
        
//-----------------------------------------------------------------------------
// name: render()
// desc: render next frame of graphics
//-----------------------------------------------------------------------------
void drawInst()
{
    setSquareColor();
    updateNoteWeight(g_mixer->getChord());
    for (int i = 0; i < g_numSquares; i++){
        g_squares[i]->render();
    }
}

//////----------
int getCurrentNote(){
    int note = -1;
    if (g_currentTrack == LEAD_TRACK || BASS_TRACK){
        for (int i = 0; i < g_numSquares; i++){
            if (g_squares[i]->isTouchWithin(g_touches[g_numActiveTouches-1]->loc)){
                note = g_squares[i]->m_keyLabel;
                return note;
            }            
        }
    }
    return note; 
}

//-----------------------------------------------------------------------------
// updateChordWeight
// changes the alpha to emphasize more likely destinations
//-----------------------------------------------------------------------------
void updateChordWeight(int currentChord){
    // loop through all of 'em 
    int hex_wide=int(ceil((g_GfxWidth/(2*g_hex_radius/sqrt(3)))));
    int hex_height=int(ceil(g_GfxHeight/((2*g_hex_radius)/sqrt(3))));
    
    for (int i=0;i<hex_wide;++i){
        for (int j=0; j<hex_height;++j){
            PulseEntity * cur= g_hexes[i*hex_height+j];
            cur->currentScaleDegree(currentChord);
        }
    }
}

//-----------------------------------------------------------------------------
// updateNoteWeight
// changes the alpha to emphasize chord tones
//-----------------------------------------------------------------------------
void updateNoteWeight(CHORD currentChord){
    
    for (int i=0;i<g_numSquares;i++){
            
        g_squares[i]->currentChord(currentChord);
    
    }
}


//-----------------------------------------------------------------------------
// name: draw_background
// desc: draws the background hexes
//-----------------------------------------------------------------------------
void drawHexBackground()
{
    if (g_currentTrack == CHORD_TRACK){
        updateChordWeight(g_mixer->getChordRootDegree());
        for (int i=0;i<g_num_hexes;++i){
            g_hexes[i]->render();
        }
    }
    
    
}

//-----------------------------------------------------------------------------
// name: find_hex
// desc: given a location, find the nearest hex
//-----------------------------------------------------------------------------
PulseEntity * find_hex(Vector3D l)
{
    for (int i=0; i<g_num_hexes; ++i){
        if ((l-g_hexes[i]->loc).magnitude()<g_hex_radius){
            //            std::cerr << i << std::endl;
            //NSLog(@"index found: %i", i);
            return g_hexes[i];
        }
    }
    NSLog(@"didn't find one");
    return NULL;
}

//////////////////////////////////
// GUI functions
//////////////////////////////////
void Record(){
    g_playMode = RECORD;
    g_mixer->Record();
    
}
void Play(){
    g_playMode = PLAY;
    g_mixer->Play();
}
void Stop(){
    g_playMode = STANDBY;
    g_mixer->Stop();
}
void Dub(){
    g_playMode = RECORD_ARTICULATION;
    g_mixer->Dub();
}

void Erase(){
    g_mixer->Erase();
}



////// sharing is nice ////////

audioRecording getAudioRecording(){
    audioRecording record;
    record.length = 0;
    if (g_currentTrack == AUDIO_TRACK || g_currentTrack == AUDIO_TRACK_2)
        record = g_mixer->getAudioRecording(g_currentTrack);
    
    return  record;
}

audioRecording getAudioRecording(int track){
    audioRecording record;
    record.length = 0;
    if (track == AUDIO_TRACK || track == AUDIO_TRACK_2)
        record = g_mixer->getAudioRecording(track);
    
    return  record;
}

synthRecording getNoteRecording(){
    synthRecording record;
    if (g_currentTrack == CHORD_TRACK || g_currentTrack == LEAD_TRACK || g_currentTrack == BASS_TRACK)
        record = g_mixer->getNoteRecording(g_currentTrack);
    
    return record;
}

synthRecording getNoteRecording(int track){
    synthRecording record;
    if (track == CHORD_TRACK || track == LEAD_TRACK || track == BASS_TRACK)
        record = g_mixer->getNoteRecording(track);
    
    return record;
}

artRecording getArtRecording(){
    return g_mixer->getArtRecording(g_currentTrack);
}

artRecording getArtRecording(int track){
    return g_mixer->getArtRecording(track);
}

void loadAudioRecording(audioRecording recordData){
    if (g_mixer->loadAudioRecording(recordData) != -1)
        NSLog(@"loaded");
    else
        NSLog(@"failed to load");
}

void loadAudioRecording(int track, audioRecording recordData){
    if (g_mixer->loadAudioRecording(track, recordData) != -1)
        NSLog(@"loaded");
    else
        NSLog(@"failed to load");
}

void loadArtRecording(artRecording recording){
    if (g_mixer->loadArtRecording(recording) != -1)
        NSLog(@"loaded");
    else
        NSLog(@"failed to load");
}

void loadArtRecording(int track, artRecording recording){
    if (g_mixer->loadArtRecording(track, recording) != -1)
        NSLog(@"loaded");
    else
        NSLog(@"failed to load");
}

void loadNoteRecording(synthRecording recording){
    if (g_mixer->loadNoteRecording(recording) != -1)
        NSLog(@"loaded notes");
    else
        NSLog(@"failed to load notes");
}

void loadNoteRecording(int track, synthRecording recording){
    if (g_mixer->loadNoteRecording(track, recording) != -1)
        NSLog(@"loaded notes");
    else
        NSLog(@"failed to load notes");
}

int getCurrentTrack(){
    return g_currentTrack;
}