//
//  Track.h
//  SSS
//
//  Created by Jeffrey Harriman on 3/8/12.
//  Copyright (c) 2012 Stanford University. All rights reserved.
//

#ifndef SSS_Track_h
#define SSS_Track_h
#include "JifDefines.h"
#include "mo_gfx.h"
#include "ADSR.h"
#include "Metro.h"
#include "ResonantLowPass.h"

class Track {
protected:
    // member variables
    int m_playMode;
    int m_length;
    int m_maxLength;
    // level
    float m_gain;
    // envelope
    stk::ADSR * m_adsr;
    // time keeper
    Metro * m_metronome;
    // key pointer
    int * m_key; 
    int m_keyName;
    
    // record data
    recordData * m_noteRec;
    recordData * m_artRec;
    
    int m_ptr;
    int m_notePtr, m_numNotes;
    int m_artPtr, m_numArts;
    
public:
    Track();
    ~Track();
    void tick(SAMPLE * buffer, int numFrames);
    SAMPLE tick();
    void keyOn();
    void keyOff();
    
    // set functions
    void setGain(float gain){m_gain = gain;};
    void setLoopLength(int numSamples);
    void setLoopLength(int measures, int beatsPer, float tempo);
    void assignMetro(Metro * theMetronome){m_metronome = theMetronome;};
    void assignKeyPtr(int * theKey){m_key = theKey;};
    void setPlayMode(int mode);
    void setTimeZero();
    void setActive(bool active){m_active = active;};

    // get
    int getPlayMode(){return m_playMode;};
    int getKeyName(){return m_keyName;}; // name (#) of key pressed to put it in current state
    synthRecording getNoteRecording();
    artRecording getArtRecording();
    void loadNoteRecording(synthRecording recording);
    void loadArtRecording(artRecording recording);
    
    // recording and playback
    void recordNote(recordData);
    void recordArticulation(recordData);
    void playArticulation();
    void playNoteData();
    
    // public member vars
    bool m_active;
};

#endif
